#!/usr/bin/perl
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

use warnings;
use strict;
use Getopt::Long;

my %options = ();
my $IFCFGFILE = "/etc/sysconfig/network/ifcfg-";
my $ROUTEFILE = "/etc/sysconfig/network/routes";
my $HOSTNAMEFILE = "/etc/HOSTNAME";
my $HOSTFILE = "/etc/hosts";
my $RESOLVFILE = "/etc/resolv.conf";


if (! (GetOptions (\%options, 'eth=s', 'ip=s', 'netmask=s', 'gateway=s',
                   'host=s', 'domain=s', 'dns=s', 'suffix=s'))) {
    exit (1); 
}
if(!defined $options{'eth'} || !defined $options{'ip'} || !defined $options{'netmask'} || !defined $options{'gateway'} || !defined $options{'host'} || !defined $options{'domain'} || !defined $options{'dns'} || !defined $options{'suffix'}) {
    print "AE: Not all options specified.\n";
    exit 1;
}

$IFCFGFILE .= $options{eth};

print "AE: Setting up interface $options{eth}...\n";
print "AE: setting IP address to $options{ip} and netmask to $options{netmask}...\n";
`echo "BOOTPROTO='static'" > $IFCFGFILE`;
`echo "IPADDR='$options{ip}'" >> $IFCFGFILE`;
`echo "MTU='1500'" >> $IFCFGFILE`;
`echo "STARTMODE='onboot'" >> $IFCFGFILE`;
`echo "NETMASK='$options{netmask}'" >> $IFCFGFILE`;

print "AE: setting default gateway to $options{gateway}...\n";
`echo "default $options{gateway} 0.0.0.0" > $ROUTEFILE`;

print "AE: setting hostname to $options{host}.$options{domain}...\n";
`echo $options{host}.$options{domain} > $HOSTNAMEFILE`;
`echo "$options{ip} $options{host}.$options{domain} $options{host}"  >> $HOSTFILE`;
`hostname $options{host}.$options{domain}`;

print "AE: setting DNS server to $options{dns} and domain suffix to $options{suffix}...\n";
`echo "nameserver $options{dns}" > $RESOLVFILE`;
`echo "search $options{suffix}" >> $RESOLVFILE`;

print "AE: Network configuration completed\n";

#print "AE: Disabling setup wizard...\n";
#`touch /opt/ibm/director/vmi/data/.wizardcompleted`;

exit 0;
