#!/usr/bin/perl -w
# %Z%%M%------- %I%- %W% %G% %U% 
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# this script should dependent on the isdvmi-mgmt rpm
use warnings;
use strict;
use Getopt::Long;
use XML::Simple;

# the var to store app setting and port 
my %app_setting=();
my $RUNTIME_DIR="/opt/ibm/director/vmi/data/networksetting/firewall/";

sub get_default_fw_setting
{
    # read the default/appends settings and merge them into one hash
    foreach my $f ('/opt/ibm/director/vmi/data/conf/default_firewall.xml', glob "/opt/ibm/director/vmi/data/conf/appends/firewall/*.xml"){
        my $xml=XMLin($f, ForceArray => 1, KeyAttr=>"app_id");
        @app_setting{keys %{$xml->{'net_app'}}} = values %{$xml->{'net_app'}};
    }
}

# return 1 indicates the app name is invalid
# return 0 indicates that the modification is successful
sub modify_fw_setting
{
    my ($iface, $app, $new_setting)=@_;
    if (! exists $app_setting{$app}) {
        $app=~ s/.name$//;
    }
    if (! exists $app_setting{$app}) {
        $app.=".name";
    }
    if (! exists $app_setting{$app}) {
        return 1;
    }
    $app_setting{$app}->{'default'}="deny";
    if ($new_setting eq "allowed") {
        $app_setting{$app}->{'default'}="allow";
    }
    # recreate the runtime setting file for this interface
    &create_runtime_file ($iface);

    # modify iptables restor file for this interface
    foreach my $f ("/etc/vmiiptables", "/etc/vmiip6tables") {
        my $bak=$f.".bak";
        rename $f, $bak;
        if (! open IPTABLES_BAK, "< $bak") {
            die "AE: cannot open iptables setting file for read.\n";
        }
        my $new_rules="";
        # read rules not for this interface
        while (my $line = <IPTABLES_BAK>){
            if (! ($line =~ /-A $iface-FIREWALL-SETTING/) ){
                $new_rules.=$line;
            }
            else{
                last;
            }
        }
        # append the new rules for this interface
        while (my ($k, $v) = each %app_setting) {
            # only add the rules to ACCEPT
            if ($v->{'default'} eq 'allow') {
                my $port_option="--dport";
                # handle the case of ping and pingv6
                if ($k eq "ping.name" ){
                    if ($f eq "/etc/vmiip6tables") {
                        next;
                    }
                    else {
                        $port_option="--icmp-type";
                    }
                }
                if ($k eq "pingv6.name" ){
                    if ($f eq "/etc/vmiiptables") {
                        next;
                    }
                    else {
                        $port_option="--icmpv6-type";
                    }
                }
		foreach my $port (@{$v->{'port'}}){
			if($port_option =~ /icmpv6/){
				$new_rules.="-A $iface-FIREWALL-SETTING -i $iface -p $port->{'type'} -j ACCEPT\n";
			}else{
				$new_rules.="-A $iface-FIREWALL-SETTING -i $iface -p $port->{'type'} -m $port->{'type'} $port_option $port->{'number'} -j ACCEPT\n";
			}
                }
            }
        }
        # continue rules for other interface
        while (my $line =<IPTABLES_BAK>) {
            if (! ($line =~ /-A $iface-FIREWALL-SETTING/) ){
                $new_rules.= $line;
            }
        }
        close IPTABLES_BAK;

        # write new rules back
        if (! open IPTABLES, "> $f") {
            die "AE: cannot open iptables setting file for write.\n";
        }
        print IPTABLES $new_rules;
        close IPTABLES;
        unlink $bak;
    }

    return 0;
}

# to create runtime file for one interface
sub create_runtime_file 
{
    my ($iface)=@_;
    my $new_setting="";
    while(my ($k, $v) = each %app_setting){
        my $allow=0;
        if($v->{'default'} eq 'allow') {
            $allow=1;
        }
	$new_setting.="$k|$allow||\n";
    }
    if (! open RUNTIME, "> $RUNTIME_DIR/$iface-FIREWALL-SETTING"){
        die "AE: cannot initialize the firewall setting files!\n";
    }
    print RUNTIME $new_setting;
    close RUNTIME;
}

sub create_fw_setting_files 
{
    `mkdir -p $RUNTIME_DIR`;
    # get network interfaces, e.g. eth0, eth1 ...
    my @ifs;
    my $interfaces=`ip link show label eth* | grep "^[0-9]:" `;
    my $index=0;
    while($interfaces=~/(eth.):/){
        $ifs[$index]=$1;
        $index++;
        $interfaces=$';
    }
	
    foreach my $iface (@ifs){
        # generate the runtime files by default/appends settings
        create_runtime_file $iface;
    }

    # generate the iptables restore file
    foreach my $fiptables("/etc/vmiiptables", "/etc/vmiip6tables") {
        my $rules="*filter
:INPUT DROP [0:0]
:FORWARD DROP [0:0]
:OUTPUT ACCEPT [0:0]
";
    	foreach my $iface (@ifs){
		$rules.=":$iface-FIREWALL-SETTING - [0:0]\n";
    	}
    	foreach my $iface (@ifs){
		$rules.="-A INPUT -i $iface -j $iface-FIREWALL-SETTING\n";
    	}

	$rules.="-A INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT\n";
	$rules.="-A INPUT -i lo -j ACCEPT\n";
	if ($fiptables eq "/etc/vmiip6tables") {
 	    $rules.="-A INPUT -m frag --fragmore -j ACCEPT\n";
	    $rules.="-A INPUT -m frag --fraglast -j ACCEPT\n";
	    # SLP UA doing a multicast will get responses from SA/DAs on
	    # port 427 to the ephemeral port used by the UA to send the request
	    $rules.="-A INPUT -p udp --sport 427 --dport 427 -j ACCEPT\n";
	    $rules.="-A INPUT -p tcp --sport 427 --dport 427 -j ACCEPT\n";
	    $rules.="-A INPUT -p udp --sport 427 --dport 14252:14328 -j ACCEPT\n";
	    $rules.="-A INPUT -p tcp --sport 427 --dport 14252:14328 -j ACCEPT\n";
	    $rules.="-A INPUT -p udp --sport 427 --dport 32768:61000 -j ACCEPT\n";
	    $rules.="-A INPUT -p tcp --sport 427 --dport 32768:61000 -j ACCEPT\n";
	    # port 546 and 547 are for dhcp6 client and server
	    $rules.="-A INPUT -p udp --dport 546:547 -j ACCEPT\n";
	    $rules.="-A INPUT -p tcp --dport 546:547 -j ACCEPT\n";
	    $rules.="-A INPUT -p tcp --tcp-flags SYN,FIN SYN,FIN -m limit --limit 10/m -j DROP\n";	
	} else {
            $rules.="-A INPUT -i mgmt0 -j ACCEPT\n";
	    $rules.="-A OUTPUT -d 10.3.0.1/32 -o mgmt0 -p tcp -m owner ! --uid-owner root -j DROP\n";
	    $rules.="-A INPUT -f -j ACCEPT\n";
	    # SLP UA doing a multicast will get responses from SA/DAs on
	    # port 427 to the ephemeral port used by the UA to send the request
	    $rules.="-A INPUT -p udp --sport 427 --dport 427 -j ACCEPT\n";
	    $rules.="-A INPUT -p tcp --sport 427 --dport 427 -j ACCEPT\n";
	    $rules.="-A INPUT -p udp --sport 427 --dport 14252:14328 -j ACCEPT\n";
	    $rules.="-A INPUT -p tcp --sport 427 --dport 14252:14328 -j ACCEPT\n";
	    $rules.="-A INPUT -p udp --sport 427 --dport 32768:61000 -j ACCEPT\n";
	    $rules.="-A INPUT -p tcp --sport 427 --dport 32768:61000 -j ACCEPT\n";
	    $rules.="-A INPUT -p tcp --tcp-flags SYN,FIN SYN,FIN -m limit --limit 10/m -j DROP\n";	
        }
        foreach my $iface (@ifs){
            while (my ($k, $v) = each %app_setting) {
                # only add the rules to ACCEPT
                if ($v->{'default'} eq 'allow') {
                    # handle the case of ping and pingv6
                    my $port_option="--dport";
                    # handle the case of ping and pingv6
                    if ($k eq "ping.name" ){
                        if ($fiptables eq "/etc/vmiip6tables") {
                            next;
                        }
                        else {
                            $port_option="--icmp-type";
                        }
                    }
                    if ($k eq "pingv6.name" ){
                        if ($fiptables eq "/etc/vmiiptables") {
                            next;
                        }
                        else {
                            $port_option="--icmpv6-type";
                        }
                    }
		    foreach my $port (@{$v->{'port'}}){
			    if($port_option =~ /icmpv6/){
				    $rules.="-A $iface-FIREWALL-SETTING -i $iface -p $port->{'type'} -j ACCEPT\n";
			    }else{
				    $rules.="-A $iface-FIREWALL-SETTING -i $iface -p $port->{'type'} -m $port->{'type'} $port_option $port->{'number'} -j ACCEPT\n";
			    }
		    }
                }
            }
        }
        $rules.="COMMIT\n\n";
        if(! open IPTABLES, "> $fiptables") {
            die "AE: cannot open $fiptables to write firewall rules!\n";
        }
        print IPTABLES $rules;
        close IPTABLES;
    }
}

# the script is only called by AE
if (@ARGV %2 != 0) {
    exit 1;
}


print "AE: Configure the firewall...\n";
my %options=@ARGV;

my $iface='';
while ( my ($k, $v) = each %options){
    if ($k eq '-interface'|| $k eq '--interface') {
        $iface=$v;
        delete $options{$k};
    }
    elsif ($v ne 'allowed' && $v ne 'notallowed'){
        my $app=$k;
        $app=~ s/^-*//;
        print "AE: Invalid setting for application \'$app\'. Only 'allowed' and 'notallowed' can be specified. Ignore it.\n";
        delete $options{$k};
    }
}

if ($iface eq '') {
    print "AE: interface not specified. Ignore the configuration!\n";
    exit 1;
}

# get the default setting first
get_default_fw_setting;

# fix the issue for defect 191787
#initialize runtime fw setting anyway
create_fw_setting_files;

while (my ($app, $v) = each %options) {
    $app=~ s/^-*//;
    print "AE: $v the application: $app\n";
    my $rc=modify_fw_setting ($iface, $app, $v);
    if ($rc != 0){
        print "AE: the application name $app is invalid. Ignore the configuration.\n";
    }
}
print "AE: Configure firewall completed.\n";
exit 0;

