#!/usr/bin/perl -w
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

use strict;
use warnings;
use Getopt::Long;

my %options=();
my $newlocale='';

if (! (GetOptions (\%options, 'locale=s'))) {
    exit (1); 
}
my $locale=$options{'locale'};
if (! defined($locale)){
    print "AE: No locale configuration needed\n";
    exit (0);
}

print "AE: Configure the locale ...\n";
# check if the input locale is valid or not
my %defaultLocale=(
    'en'=>'en_US',
    'fr'=> 'fr_FR',
    'de'=>'de_DE',
    'it'=>'it_IT',
    'ja'=>'ja_JP',
    'ko'=>'ko_KR',
    'es'=>'es_ES'
);

my @key=keys%defaultLocale;
foreach my $l (@key){
    if ($locale eq $l){
        $newlocale=$defaultLocale{$l};
        last;
    }
}
my @locales;
if ($newlocale eq ''){
    # get all avaliable locales
    my $validLocales=`/usr/smrshbin/lsconfig -L`;
    chomp($validLocales);
    $validLocales=~s/^locales=//;
    @locales=split/:/,$validLocales;
    foreach my $l (@locales){
        if ($locale eq $l){
            $newlocale=$l;
        }
    }
}
if ($newlocale ne ''){
    system ("/usr/smrshbin/chconfig -l $newlocale");
    print "AE: Locale configuration completed. New locale is $newlocale\n";
    exit (0);
}
else{
    print "AE: Invalid locale to set. The valid locales are:";
    my $c=0;
    foreach my $l (@locales){
        if($c%4==0) {print "\n    ";}
        printf "%-20s",$l;
        $c++;
    }
    print "\n";
    print "AE: Locale configuration failed.\n";
    exit (1);
}

