# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2012 All Rights Reserved 
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996 
#
#
#####################################################
# This file contains all the common declarations
# It will also load any common code
#####################################################

if [ -z "$CMD_DIR" ]; then
	echo "Internal error: CMD_DIR must be set."
	exit 1
fi

# Pass in the mandatory common code to load
load_common_code () {
    local code_file=${1}
    if [ -e $code_file ]; then
        source $code_file
    else
        echo "Internal error: Common code not found: $code_file"
        exit 1
    fi
}

#
RPATH=/usr/bin:/bin/:/sbin/:/opt/ibm/director/bin:/usr/smrshbin/:
export PATH=$RPATH:$PATH

## Configuration Section ############
DFT_DIRECTOR_INSTALL=/opt/ibm/director
### -------------------------------------------------------
# Call set_director_dirs() to set these directories
INSTALL_DIR=        # The Director install directory
JREBIN_DIR=			# The JRE bin directory (for keytool)
SMHA_DIR=           # The HA directory in the install path
SMHA_CONFIGDIR=     # The config directory in the HA path where the SSL key material is stored
SMHA_SCRIPTS=       # Where the HA scripts are located
SMHA_INCLUDE=       # Where the HA includes are located
SMHA_APP_RSRC_CMDS= # where the start/stop/monitor commands are located
SMHA_CFG_FILE=      # The full path to the HA Config file
BACKUP_DIR=         # Where we backup Director's data
AGENTMANAGER_DIR=   # Where the Agent Mgr files are located
CASMANAGER_DIR=     # Where the CAS Agent files are located
LOG_DIR=            # The log directory (for compatibility)
LOG=                # The log file (for compatibility)
CFG_STATE_FILE=     # The configuration state
DIR_OVERRIDES_DIR=  # The Director conf overrides directory
DIR_SECURITY_DIR=   # The Director LWI security directory

### -------------------------------------------------------

NONVMI_CFG=0

SMHA_CFG_FILENAME=smhacfg.prop

# Debug & logging settings
# To turn debug on, set it to ON; this writes debug to stdout
SMHA_DEBUG=O
SMHA_DEBUG_PGM_ONLY=
#SMHA_DEBUG_PGM_ONLY=smhatsadir.sh
SMHA_DEBUG_TSA=ON
SMHA_LOG_VERBOSE=0        # set to "verbose" to log via smha_debug_verbose

# source the base definitions now, after the SMHA_DEBUG* variables are set.
# the base definitons are needed for the following defines.

load_common_code /opt/ibm/director/ha/bin/include/smhabasedef.sh

# Mount point for the non-appliance
HA_MOUNTPOINT=/mnt/smha

# DB Settings
DB2HAICU=0
DB2INST=${_SMHA_DB2INST}
DB2ADM=db2iadm1
DB2DB=IBMDIR01
DB2PART=${_SMHA_DB2PART}
eval DB2INST_HOME=~${DB2INST}
DB2DATA_DIR=$DB2INST_HOME/$DB2INST
DB2SQL_DIR=$DB2INST_HOME/sqllib

# Quorum Values
NET_QUORUM=netq
Y_QUORUM=yq

RSCT_NODEINFO_FILE=/var/ct/cfg/ctsec.nodeinfo
RSCT_NODEID_FILE=/etc/ct_node_id
RSCT_NETMON_CF_FILE=/var/ct/cfg/netmon.cf

RSCT_CRIT_RSRC_PROTECT__HARD_RESET=1
RSCT_CRIT_RSRC_PROTECT__OFF=7


TSA_DOMAIN=director
TSA_RETRY_COUNT=3 
STORAGE_RG=storage-rg
STORAGE_RG_PRIORITY=15
DRBD_RG=${SMHA_DRBD_RG}
DRBD_RG_PRIORITY=30
DIRECTOR_RG=${SMHA_DIRECTOR_RG}
DIRECTOR_RG_PRIORITY=60
NFS_RSRC_NAME=nfs
DIRECTOR_RSRC_NAME=${SMHA_DIRECTOR_RSRC_NAME}
DIRECTOR_MONITOR_SCRIPT=   # defined below
#ADDRGMBR_ORDER="O,Failback"
ADDRGMBR_ORDER="O"

ADVANCED_MGR_RG_PRIORITY=45

DB2_RG=${SMHA_DB2_RG}
DB2_RG_PRIORITY=45
DB2_DB_RSRC_NAME=${SMHA_DB2_DB_RSRC_NAME}
DB2_MON_CMD_PERIOD=60
DB2_MON_CMD_TIMEOUT=60
DB2_MON_START_TIMEOUT=360
DB2_MON_STOP_TIMEOUT=360
DB2_MIRROR_NAME=mirror0

CIM_RG=${SMHA_CIM_RG}
CIM_RSRC_NAME=${_SMHA_CIM_RSRC_NAME_PFX}
#CIM_RG_PRIORITY=45
CIM_MONITOR_SCRIPT=   # defined below
CIM_MON_CMD_PERIOD=20
CIM_MON_CMD_TIMEOUT=60
CIM_MON_START_TIMEOUT=120
CIM_MON_STOP_TIMEOUT=120


CLEANUP_COMMAND_SCRIPT=tsa/smhatsaclnup.sh
CLEANUP_COMMAND_TIMEOUT=120

DIR_MON_CMD_PERIOD=10
DIR_MON_CMD_TIMEOUT=60
#DIR_MON_START_TIMEOUT=3000
DIR_MON_START_TIMEOUT=2400
DIR_MON_STOP_TIMEOUT=600
ALIAS_RSRC_NAME=alias
ALIAS6_RSRC_NAME=aliasv6

NTP_RG=${SMHA_NTP_RG}
NTP_RSRC_NAME=${SMHA_NTP_RSRC_NAME}
NTP_MONITOR_SCRIPT=   # defined below
NTP_MON_CMD_PERIOD=30
NTP_MON_CMD_TIMEOUT=20
NTP_MON_START_TIMEOUT=20
NTP_MON_STOP_TIMEOUT=20

DHCP_RG=${SMHA_DHCP_RG}
DHCP6_RG=${SMHA_DHCP6_RG}
DHCP_RSRC_NAME=${SMHA_DHCP_RSRC_NAME}
DHCP6_RSRC_NAME=${SMHA_DHCP6_RSRC_NAME}
DHCP_MONITOR_SCRIPT=   # defined below
DHCP_MON_CMD_PERIOD=60
DHCP_MON_CMD_TIMEOUT=20
DHCP_MON_START_TIMEOUT=120
DHCP_MON_STOP_TIMEOUT=120
DHCP_MIRROR_NAME=mirror0


HADM_RG=hadaemon-rg
HADM_RSRC_NAME=hadaemon
HADM_MONITOR_SCRIPT=tsa/smhatsadm.sh
HADM_MON_CMD_PERIOD=20
HADM_MON_CMD_TIMEOUT=20
HADM_MON_START_TIMEOUT=20
HADM_MON_STOP_TIMEOUT=20

HAMQ_RSRC_NAME=hamq
HAMQ_MONITOR_SCRIPT=tsa/smhatsamq.sh
HAMQ_MON_CMD_PERIOD=20
HAMQ_MON_CMD_TIMEOUT=20
HAMQ_MON_START_TIMEOUT=20
HAMQ_MON_STOP_TIMEOUT=20

# Change this to 1 to use drbd equivalences rather than locks
DRBD_EQU=1

GETMIRROR_CMD=/opt/ibm/director/vmi/bin/getmirror
GETMIRROR_DEPENDENCY_CMD=/opt/ibm/director/vmi/bin/getmirrordep

DRBDB_RSRC_NAME=${SMHA_DRBDB_RSRC_NAME}
DRBDP_RSRC_NAME=${_SMHA_DRBDP_RSRC_NAME_PFX}
DRBDS_RSRC_NAME=${_SMHA_DRBDS_RSRC_NAME_PFX}
DRBDM_RSRC_NAME=${_SMHA_DRBDM_RSRC_NAME_PFX}
DB2MIRROR_RSRC_NAME=${DRBDM_RSRC_NAME}-${DB2_MIRROR_NAME}
DHCPMIRROR_RSRC_NAME=${DRBDM_RSRC_NAME}-${DHCP_MIRROR_NAME}
DRBD_RSRC_NAME_SEARCH=drbd%
DRBD_MONITOR_SCRIPT=   # defined below
DRBD_MON_CMD_PERIOD=30
DRBD_MON_CMD_TIMEOUT=60
DRBD_MON_START_TIMEOUT=300
DRBD_MON_STOP_TIMEOUT=300
DRBD_CONF=/etc/drbd.conf
DRBD_INVALIDATE_FILE=drbd-invalid

UPDATESLIB_RSRC_NAME=updateslib
UPDATESLIBMIRROR_RSRC_NAME=${DRBDM_RSRC_NAME}-${UPDATESLIB_RSRC_NAME}


# Director Values
DIR_SHUTDOWN_DELAY=10

# Config State
CFG_STATE_NONE=NO_CONFIG
CFG_STATE_NODE=NODE_CONFIGURED
CFG_STATE_TSA=TSA_MODEL_STARTED
CFG_STATE_FINISHED=FINISHED
CFG_STATE_FILE_NAME=smhastate

HA_TAR_FILE_NAME_PREFIX=smha

### Functions ######################

# Setup the HA path and filenames
set_director_dirs () {
        
    # Find the install directory.  Assume the scripts are always run from the install path
    if [ -z "$INSTALL_DIR" ]; then
        INSTALL_DIR=${CMD_DIR%/ha/bin*}
    fi
        
    SMHA_DIR=$INSTALL_DIR/ha
    SMHA_SCRIPTS=$SMHA_DIR/bin
    SMHA_APP_RSRC_CMDS=${SMHA_SCRIPTS}/tsa
    SMHA_INCLUDE=$SMHA_SCRIPTS/include
    SMHA_CONFIGDIR=$SMHA_DIR/config
    SMHA_CFG_FILE=$SMHA_CONFIGDIR/$SMHA_CFG_FILENAME
    BACKUP_DIR=$SMHA_DIR/data-bkup
    AGENTMANAGER_DIR=$INSTALL_DIR/lwi/runtime/agentmanager
    CASMANAGER_DIR=$AGENTMANAGER_DIR/eclipse/plugins/com.ibm.tivoli.cas.manager
    AGENT_DIR=$INSTALL_DIR/agent
    CFG_STATE_FILE=$SMHA_DIR/$CFG_STATE_FILE_NAME
    JREBIN_DIR=$INSTALL_DIR/jre/bin
    
    LOG_DIR=${_SMHA_LOG_DIR}
    LOG=${_SMHA_LOG}
    
    HA_DIRECTOR_DIR=$HA_MOUNTPOINT/director
    HA_AGNTMGR_DIR=$HA_MOUNTPOINT/agentmanager
    HA_DB2DIR=$HA_DIRECTOR_DIR/db2
	
    USMIKERNEL_PROPS=$INSTALL_DIR/lwi/conf/overrides/USMIKernel.properties
    IPPREFERENCE_PROPS=$INSTALL_DIR/data/IPPreference.properties
    HA_PROPS=$INSTALL_DIR/lwi/conf/overrides/ha.properties
    TWGNEW_PROP_FILE=$INSTALL_DIR/agent/conf/overrides/twgnew.properties
    AUTOMATION_FLAGFILE=$SMHA_CONFIGDIR/.smhaEnableAutomation
    SYNCCONFIG_FLAGFILE=$SMHA_CONFIGDIR/.smhaTSASyncConf
    REFRESH_MANAGEMENTIP_FLAGFILE=$SMHA_CONFIGDIR/.refreshManagementIP
    EXCLUDE_PEER_FLAGFILE=$SMHA_CONFIGDIR/.smhaPeerNodeExcluded
    SHUTDOWN_HALT_FLAGFILE=$SMHA_CONFIGDIR/.smhaNodeHalt
    STOP_MAINT_FLAGFILE=$SMHA_CONFIGDIR/.smhaStopMaint
    DM_CONF=$SMHA_CONFIGDIR/smhadm.conf
    CFG_STATUS_FILE=$SMHA_CONFIGDIR/.cfgstatus
    ERROR_DURING_CONFIG_FLAGFILE=$SMHA_CONFIGDIR/.errorDuringConfig
    BROKER_KS=$SMHA_CONFIGDIR/broker.ks
    EMULATE_RSRCS=$SMHA_CONFIGDIR/smha_emulation

    BKRT_NO_HASTOP_FLAGFILE=$SMHA_DIR/plugins/conf/.backupNoHAStop
    BKRT_FLAGFILE=$SMHA_CONFIGDIR/.smhabackup
    BKRT_CTBACKUP_OUT=${LOG_DIR}/ctbackup.out
    BKRT_CTRESTORE_OUT=${LOG_DIR}/ctrestore.out
    
    HACMD_INPROGRESS=$SMHA_CONFIGDIR/.smhaCmdInProgress

    RESTART_AUTOMATION_FLAGFILE=$SMHA_CONFIGDIR/.smharestartAutomation
    RESTART_RSCT_FLAGFILE=$SMHA_CONFIGDIR/.smharestartRSCT

    SYSLOG_TRIP_FILE=$SMHA_CONFIGDIR/.syslog.stopped
    SYSLOG_STOPPED_DHCP=$SMHA_CONFIGDIR/.syslog.stopped.dhcp
    DIR_OVERRIDES_DIR=$DFT_DIRECTOR_INSTALL/lwi/conf/overrides
    DIR_OVERRIDES_TMP="/tmp/__overrides__"
    LWI_SECURITY_DIR=$DFT_DIRECTOR_INSTALL/lwi/security
    LWI_SECURITY_TMP="/tmp/__lwi_security__"
    DIR_SECURITY_DIR=$DFT_DIRECTOR_INSTALL/data/security
    DIR_SECURITY_TMP="/tmp/__security__"
    DIR_AMCERTS_DIR=$DFT_DIRECTOR_INSTALL/lwi/runtime/agentmanager/eclipse/plugins/com.ibm.tivoli.cas.manager/certs
    DIR_AMCERTS_TMP="/tmp/__amcerts__"
}

# the following function is to work around a TSA bug in the chrg command.
# It may return an error message indicating a failure, but the exit code is 0.
# This function will verify that the expected change has occurred before returning
# a good exit code. The function will make 3 attempts, 5 seconds apart
#
# The function takes three or five arguments, depending on the first argument value.
#
#   Arg1 = -o
#   Arg2 = online | offline     (not case sensitive, unlike chrg command)
#   Arg3 = resource group name
#
#   Arg1 = -l
#   Arg2 = None | Collocated    (not case sensitive, unlike chrg command)
#   Arg3 = resource group name
#
#   Arg1 = -u
#   Arg2 = a | d
#   Arg3 = -x
#   Arg4 = node name
#   Arg5 = resource group name
#
# The caller is expected to capture stdout from this function, which includes stderr from chrg
#
# Function Exit status:  0  success
#                        99 could not effect change after three attempts
#                        125 missing required argument
#                        N  any other value is from chrg

call_tsa_chrg_o_l_u() {
    local lsArg=
    local ExpectedOutput=
    local RGname=
    local MatchNode=
    local tsa_result=
    local rc=
    local lsoutput=

    if [ $# -lt 3 ]
    then
        # The caller has a bug; must be at least three args
        return 125
    fi

    case $1 in
        -o )
            lsArg=NominalState
            if [[ $2 = "online" ]]
            then
                ExpectedOutput=0
            else
                ExpectedOutput=1
            fi
            RGname=$3
        ;;

        -l )
            lsArg=MemberLocation
            if [[ $2 = "None" ]]
            then
                ExpectedOutput=0
            else
                ExpectedOutput=1
            fi
            RGname=$3
        ;;

        -u )
            if [ $# -ne 5 ]
            then
                # The caller has a bug; must be five args
                return 125
            fi

            if [ $2 = "a" ]
            then
                ExpectedRC=0
            else
                ExpectedRC=1
            fi
            MatchNode=$4
            RGname=$5

        ;;

        * )
            # The caller has a bug; 1st arg must be of of -o, -l or -u
            return 125

        ;;
    esac

    for (( i=3; i > 0; i-- ))
    do
        tsa_result=$(chrg $@ 2>&1)
        rc=$?

        if [[ $rc -ne 0 ]]
        then
            if (( i > 1 ))
            then
                sleep 5
                continue
            fi

            echo "$tsa_result"
            return $rc
        fi

        if [[ -n $ExpectedOutput ]]
        then
            lsoutput=$(LC_ALL=C lsrsrc-api -s "IBM.ResourceGroup::Name == \"$RGname\"::${lsArg}" 2> /dev/null)
            if [[ $lsoutput = $ExpectedOutput ]]
            then
                echo "$tsa_result"
                return 0
            fi
        else
            # cannot use lsrsrc-api, since that returns a resource handle in ExcludedList

            LC_ALL=C lsrg -g $RGname | grep "ExcludedList.*${MatchNode}" > /dev/null 2>&1; rc=$?
            if [[ $ExpectedRC = $rc ]]
            then
                echo "$tsa_result"
                return 0
            fi
        fi

        if (( i > 1 ))
        then
            sleep 5
        fi
    done

    echo "$tsa_result"
    return 99
}

# the following function is to work around a TSA bug in the samctrl command.
# It may return an error message indicating a failure, but the exit code is 0.
# This function will verify that the expected change has occurred before returning
# a good exit code. The function will make 3 attempts, 5 seconds apart
#
# The function takes one or two arguments.
#
#   Arg1 = the option value to be passed with the -M flag of samctrl
#   Arg2 = if present, the option value to be passed with the -r flag of samctrl
#
# The caller is expected to capture stdout from this function, which includes stderr from samctrl
#
# Function Exit status:  0  success
#                        99 could not effect change after three attempts
#                        125 missing required argument
#                        N  any other value is from samctrl

call_tsa_samctrl_M_r() {
    local MFlagArg=
    local rFlagOpt=
    local lsArg=
    local ExpectedOutput=
    local tsa_result=
    local rc=
    local lsoutput=

    if [ $# -lt 1 ]
    then
        # The caller has a bug; must be at least one arg
        return 125
    fi

    MFlagArg=$1
    if [[ $MFlagArg = "T" ]]
    then
        ExpectedOutput=0
    else
        ExpectedOutput=1
    fi

    if [ $# -gt 1 ]
    then
        ExpectedOutput=${ExpectedOutput}::$2

        rFlagOpt="-r $2"
        lsArg="::RetryCount"
    fi

    # echo "MFlagArg=$MFlagArg  rFlagOpt=$rFlagOpt  ExpectedOutput=$ExpectedOutput"

    for (( i=30; i > 0; i-- ))
    do
        tsa_result=$(samctrl -M $MFlagArg $rFlagOpt 2>&1)
        rc=$?

        if [[ $rc -ne 0 ]]
        then
            if (( i > 1 ))
            then
                sleep 5
                continue
            fi

            echo "$tsa_result"
            return $rc
        fi

        lsoutput=$(LC_ALL=C lsrsrc-api -c IBM.CHARMControl::::Automation${lsArg} 2> /dev/null)
        if [[ $lsoutput = $ExpectedOutput ]]
        then
            echo "$tsa_result"
            return 0
        fi

        if (( i > 1 ))
        then
            sleep 5
        fi
    done

    echo "$tsa_result"
    return 99
}

export TEXTDOMAINDIR=/usr/share/locale
export TEXTDOMAIN=smhacmd
. /usr/bin/gettext.sh

### Main Code ######################

HOST=`hostname -s 2>/dev/null`
if [ $? != 0 ]; then
    HOST=`hostname`
fi

set_director_dirs

# Check for override of directory where monitor scripts (see below)
# can be found.

SMHA_EMULATED_RSRCS=      # no resource emulation

if [ -f $EMULATE_RSRCS -a -s $EMULATE_RSRCS ]
then
    read Dirname junk < $EMULATE_RSRCS

    if [ -n "$Dirname" -a -d "$Dirname" ]
    then
        SMHA_APP_RSRC_CMDS=$Dirname
        SMHA_EMULATED_RSRCS=on
    fi
    unset Dirname
    unset junk
fi

# Define "monitor scripts" => the start/stop/monitor commands
# specified on IBM.Application resources
#
# N.B. except for DB2, the same script is used for all three purposes

ARP_MONITOR_SCRIPT=${SMHA_APP_RSRC_CMDS}/smhatsaarp.sh
CIM_MONITOR_SCRIPT=${SMHA_APP_RSRC_CMDS}/smhatsacim.sh
DHCP_MONITOR_SCRIPT=${SMHA_APP_RSRC_CMDS}/smhatsadhcp.sh
DIRECTOR_MONITOR_SCRIPT=${SMHA_APP_RSRC_CMDS}/smhatsadir.sh
DRBD_MONITOR_SCRIPT=${SMHA_APP_RSRC_CMDS}/smhatsadrbd.sh
NTP_MONITOR_SCRIPT=${SMHA_APP_RSRC_CMDS}/smhatsantp.sh

DB2_START_SCRIPT=${SMHA_APP_RSRC_CMDS}/db2_start.ksh
DB2_STOP_SCRIPT=${SMHA_APP_RSRC_CMDS}/db2_stop.ksh
DB2_MONITOR_SCRIPT=${SMHA_APP_RSRC_CMDS}/db2_monitor.ksh

# modify timeouts for emulation

if [ -n "$SMHA_EMULATED_RSRCS" ]
then
    CIM_MON_START_TIMEOUT=$(( CIM_MON_CMD_PERIOD * 2 ))
    DB2_MON_START_TIMEOUT=$(( DB2_MON_CMD_PERIOD * 2 ))
    DHCP_MON_START_TIMEOUT=$(( DHCP_MON_CMD_PERIOD * 2 ))
    DIR_MON_START_TIMEOUT=$(( DIR_MON_CMD_PERIOD * 2 ))
    DRBD_MON_START_TIMEOUT=$(( DRBD_MON_CMD_PERIOD * 2 ))
    NTP_MON_START_TIMEOUT=$(( NTP_MON_CMD_PERIOD * 2 ))

    TSA_RETRY_COUNT=1 
fi

load_common_code $SMHA_INCLUDE/smhacomcode.sh

# ensure proper umask for all scripts

umask 022

mk_log_dir

#debug "$PROGNAME COMMON PATH:$PATH"

