# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#

DIR_START_SERVICE="/opt/ibm/director/vmi/bin/dirstartservice"
DIR_START_FLAG_FILE="$SMHA_CONFIGDIR/.dirsinittab"

crt_dir_rsrc () {
    # This should have been caught already, but just in case
    ensure_parms "node"
    
    # Only create the resource if it doesn't already exist
    lsrg -A d -g $DIRECTOR_RG &>/dev/null
    if [ $? -eq 6 ]; then
        debug "Create the director resources"
        mkrsrc IBM.Application \
        Name="$DIRECTOR_RSRC_NAME" \
        StartCommand="$DIRECTOR_MONITOR_SCRIPT start" \
        StopCommand="$DIRECTOR_MONITOR_SCRIPT stop" \
        MonitorCommand="$DIRECTOR_MONITOR_SCRIPT status" \
        MonitorCommandPeriod=$DIR_MON_CMD_PERIOD \
        MonitorCommandTimeout=$DIR_MON_CMD_TIMEOUT \
        NodeNameList="{'$HOST','$node'}" \
        StartCommandTimeout=$DIR_MON_START_TIMEOUT \
        StopCommandTimeout=$DIR_MON_STOP_TIMEOUT \
        UserName="root" \
        ResourceType=1
        
#        CleanupNodeList="{'$HOST','$node'}" \
#        CleanupCommand="$SMHA_SCRIPTS/$DIRECTOR_MONITOR_SCRIPT cleanup" \
#        CleanupCommandTimeout=$CLEANUP_COMMAND_TIMEOUT \
        
        # create the resource group
        mkrg -p $DIRECTOR_RG_PRIORITY $DIRECTOR_RG
        add_to_rg $DIRECTOR_RSRC_NAME IBM.Application $DIRECTOR_RG $ADDRGMBR_ORDER
        take_rg_offline_n_wait $DIRECTOR_RG
        
    else
        debug "director resources already created"   
    fi  
}

crt_dir_dep () {
	
    debug "Create the director dependencies"

    # Director depends on DB2 DB
    make_relationship DependsOn IBM.Application:$DIRECTOR_RSRC_NAME \
        IBM.Application:${DB2_DB_RSRC_NAME} $DIRECTOR_RSRC_NAME-depon-${DB2_DB_RSRC_NAME}    
 	
 	# Director depends on updateslib
 	make_relationship DependsOn IBM.Application:$DIRECTOR_RSRC_NAME \
 		IBM.Application:${UPDATESLIBMIRROR_RSRC_NAME} $DIRECTOR_RSRC_NAME-depon-${UPDATESLIB_RSRC_NAME}   
}

remove_director_autostart () {
#    debug "Remove director auto start"
    /sbin/chkconfig -d smserver &>/dev/null
}

remove_dirstartservice () {
	if [ -x $DIR_START_SERVICE ]; then
		$DIR_START_SERVICE remove
	fi
}

add_dirstartservice () {
    if [ -x $DIR_START_SERVICE ]; then
        $DIR_START_SERVICE add
    fi
}

save_dirstartservice_status () {
	local dirstat=`$DIR_START_SERVICE status`
	if [[ "$dirstat" == "Yes" ]]; then
		touch $DIR_START_FLAG_FILE
	fi
}

restore_dirstartservice_status () {
	if [ -e $DIR_START_FLAG_FILE ]; then
		add_dirstartservice
		rm -f $DIR_START_FLAG_FILE
	fi
}

#comment_dirs_inittab () {
#    perl -pi -e "s/^dirs:2345/#dirs:2345/g" /etc/inittab
#}
#uncomment_dirs_inittab () {
#    perl -pi -e "s/^#dirs:2345/dirs:2345/g" /etc/inittab
#}

#The two propertity files' details are in the following:
#1、/opt/ibm/director/vmi/data/dirc.properties
#active-node-url=https://localhost:8422/ibm/console/logon.jsp
#active-node-wait-for-director=yes
#passive-node-url=file://some/file.htm
#passive-node-wait-for-director=no
#browser_pid=12345 # this line records process id of the browser
#2、/opt/ibm/director/vmi/data/dirc-url-in-use.properties
#url=https://localhost:8422/ibm/console/logon.jsp
#wait-for-director=yes

DIRC_MASTER_FILE=/opt/ibm/director/vmi/data/dirc.properties
DIRC_OEM_MASTER_FILE=/opt/ibm/director/vmi/data/overwrite/dirc.properties
DIRC_ACTIVE_FILE=/opt/ibm/director/vmi/data/dirc-url-in-use.properties

get_dirc_browser_pid () {
	local pid
	pid=`grep browser_pid $DIRC_ACTIVE_FILE | sed "s/=/ /g" | awk '{print $2}'`
	debug "DIRC browser pid:$pid"
	DIRC_BROWSER_PID=$pid
}

kill_dirc_browser () {
	#debug "Kill dirc browser pid:$DIRC_BROWSER_PID"
	#kill $DIRC_BROWSER_PID
	ps auxww | grep "^cons" | grep "/usr/lib/firefox" | awk '{print $2}' | xargs kill
}

dirc_use_settings () {
	# Passed value must be "active" or "passive"
	local which_node=$1

	if [ -f $DIRC_OEM_MASTER_FILE -a -s $DIRC_OEM_MASTER_FILE ]; then
	    masterfile=$DIRC_OEM_MASTER_FILE
	else
	    masterfile=$DIRC_MASTER_FILE
	fi
	grep "${which_node}-node-" $masterfile | sed "s/${which_node}-node-//g" >$DIRC_ACTIVE_FILE
}



