# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#

run_init () {

    TWG_DIRECTORY_ROOT="/opt/ibm/director"
    TWG_DIRECTORY_BIN="$TWG_DIRECTORY_ROOT/bin"
    TWG_DIRECTORY_DATA="$TWG_DIRECTORY_ROOT/data"
    TWG_DIRECTORY_PRODDATA="$TWG_DIRECTORY_ROOT/proddata"

    CAS_SYSPROPS="-Dtwg.directory.proddata=$TWG_DIRECTORY_PRODDATA"
    CAS_SYSPROPS="$CAS_SYSPROPS -Dtwg.directory.root=$TWG_DIRECTORY_ROOT"
    CAS_SYSPROPS="$CAS_SYSPROPS -Dagent.directory.root=$TWG_DIRECTORY_ROOT/agent"

    SMHA_CLASS=com.ibm.vmi.ha.Initializer
    SM_JAVA=/opt/ibm/director/jre/bin/java
    
    sshPath=`find /opt/ibm/director/lwi/runtime/USMi*/eclipse/plugins/com\.ibm\.net\.ssh* -name *ssh*\.jar | sort -r | head -n 1 -`
    SM_CLASSPATH=${sshPath}
    
    slpPath=`find /opt/ibm/director/lwi/runtime/USMi*/eclipse/plugins/com\.ibm\.usmi\.services* -name *slp*.jar | sort -r | head -n 1 -`
    SM_CLASSPATH=$SM_CLASSPATH:${slpPath}
    
    remoteaccessPath=`find /opt/ibm/director/lwi/runtime/USMi*/eclipse/plugins/com\.ibm\.tivoli\.remoteaccess* -name *remoteaccess*.jar | sort -r | head -n 1 -`
    SM_CLASSPATH=$SM_CLASSPATH:${remoteaccessPath}
    
    base64path=`find /opt/ibm/director/lwi/runtime/core/eclipse/plugins/ -name org\.apache\.commons\.codec_*\.jar | sort -r | head -n 1 -`
    SM_CLASSPATH=$SM_CLASSPATH:${base64path}
        
    coreKernelPath=`find /opt/ibm/director/lwi/runtime/USMi*/eclipse/plugins/com\.ibm\.director\.core\.kernel* -name core-kernel.jar | sort -r | head -n 1 -`
    SM_CLASSPATH=$SM_CLASSPATH:${coreKernelPath}
    
    commonsLoggingPath=`find /opt/ibm/director/lwi/runtime/core/eclipse/plugins/ -name org\.apache\.commons\.logging_*\.jar | sort -r | head -n 1 -`
    SM_CLASSPATH=$SM_CLASSPATH:${commonsLoggingPath}
        
    cmdAPIPath=`find /opt/ibm/director/lwi/runtime/ApplianceMgmt/eclipse/plugins/com\.ibm\.vmi\.cmdapi\.util* -name CmdAPI.jar | sort -r | head -n 1 -`
    SM_CLASSPATH=$SM_CLASSPATH:${cmdAPIPath}
    
    jsonPath=`find /opt/ibm/director/lwi/runtime/rest/eclipse/plugins/ -name *wink-json4j*\.jar | sort -r | head -n 1 -`
    SM_CLASSPATH=$SM_CLASSPATH:${jsonPath}

    activeMQPath=`find /opt/ibm/director/lwi/runtime/messaging/eclipse/plugins/org.apache.activemq_* -name *activemq-all*\.jar | sort -r | head -n 1 -`
    SM_CLASSPATH=$SM_CLASSPATH:${activeMQPath}

    SM_CLASSPATH=$SM_CLASSPATH:/opt/ibm/director/ha/lib/classes
    SM_CLASSPATH=$SM_CLASSPATH:/opt/ibm/director/ha/lib/smhadm.jar
    SM_CLASSPATH=$SM_CLASSPATH:$TWG_DIRECTORY_ROOT/lwi/lib/passutils.jar
    
    
    SM_LOGGING=-Djava.util.logging.config.file=/opt/ibm/director/ha/logs/init_logging.properties
    
    if [ -f /opt/ibm/director/ha/debug ]; then
        DEBUG_ARGS="-Dsmhawait=60000 -Xdebug -Xnoagent -Xrunjdwp:transport=dt_socket,server=y,address=8000,suspend=n"
        VERBOSE="-verbose:class"
    fi
    
    # Start the Initializer
    export CLASSPATH=$SM_CLASSPATH
    $SM_JAVA $VERBOSE $DEBUG_ARGS $SM_LOGGING $CAS_SYSPROPS $SMHA_CLASS $*

    return $?
    
}

send_conf_parms () {
    
        local rc=0
        # Send the config over to the other node
        # This will do the secondary setup work
        # setup the parameters to send
        local parms
        # Send over our node name
        parms="node $HOST"
        if [ -n "$dhcp" ]; then
            parms="$parms dhcp $dhcp"
        fi
        if [ -n "$local_replication_address" ]; then
            parms="$parms local-replication-address $local_replication_address"
        fi
        if [ -n "$remote_replication_address" ]; then
            parms="$parms remote-replication-address $remote_replication_address"
        fi
        if [ -n "$agent_manager_address" ]; then
            parms="$parms agent-manager-address $agent_manager_address"
        fi
        if [ -n "$network_quorum" ]; then
            parms="$parms network-quorum $network_quorum"
        fi
        if [ -n "$activations" ]; then
            parms="$parms activations $activations"
        fi
        if [ -n "$old_hostname" ]; then
            parms="$parms old-hostname $old_hostname"
        fi
        
        # Handle multiple network alias
        local i
        for i in {1..20}
        do
            # try this name
            MULTI_PARM_VAR=network_alias__${i}
            # if we can dereference it and the length is non zero then it's set
            if [ -n "${!MULTI_PARM_VAR}" ]; then
                parms="$parms network-alias ${!MULTI_PARM_VAR}"
            fi 
        done
        # Handle multiple network alias6
        local i
        for i in {1..20}
        do
            # try this name
            MULTI_PARM_VAR=network_alias6__${i}
            # if we can dereference it and the length is non zero then it's set
            if [ -n "${!MULTI_PARM_VAR}" ]; then
                parms="$parms network-alias6 ${!MULTI_PARM_VAR}"
            fi 
        done
                        
        debug "calling init conf with parms:$parms"
        run_init "conf"  $parms
        rc=$?
        
        return $rc
    
}

