# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END


delete_tsa_model () {
    
    debug "Deleting the tsa model."

    is_domain_setup
    if [ $? -eq 0 ]; then
        if [ -f $SMHA_CFG_FILE ]; then
            # if our config file still exists just erase it and exit
            debug "The HA configuration appears to be already terminated, but config file is here, so continuing with removal of the configuration file."
            sm_msg "`eval_gettext "Removing configuration file."`"
            
            rm $SMHA_CFG_FILE
            return
        else      
            # Otherwise, neither the domain nor our config file were there.     
            # TRANSLATORS: This message is shown when the user tries to remove High Availability
            # but it is not configured
            sm_msg "`eval_gettext "High availability is not configured, so there is no need to remove it."`"
            return
        fi
    fi
        
    ### Delete the model!
    
    take_all_rg_offline
    
    run_activation_plugins deactivate 
    run_controlled_plugins delete
    
    run_plugins 6 delete
    
    is_rg_created $DIRECTOR_RG
    if [ $? -eq 1 ]; then
        # (OEMB)
        sm_msg "`eval_gettext "Waiting for the management server to shut down."`"
        take_rg_offline_n_wait_nonfatal $DIRECTOR_RG
    fi
             
    # TRANSLATORS: The "resources" specified in the message refer to hardware and
    # software components such as disks, memory, software programs such as databases,
    # ftp servers, CIM servers, etc.  These resources are being removed from the
    # high availability model.
    sm_msg "`eval_gettext "Removing the resources."`"
    
    run_plugins 4 delete

    # TRANSLATORS: The "resources" specified in the message refer to hardware and
    # software components such as disks, memory, software programs such as databases,
    # ftp servers, CIM servers, etc.  These resources are being removed from the
    # high availability model.
    sm_msg "`eval_gettext "Removing DB2 resources."`"                           
    dlt_db2_rsrc        

    run_plugins 2 delete
    
    # TRANSLATORS: The "resources" specified in the message refer to hardware and
    # software components such as disks, memory, software programs such as databases,
    # ftp servers, CIM servers, etc.  These resources are being removed from the
    # high availability model.
    sm_msg "`eval_gettext "Removing storage resources."`"
    dlt_drbd_rsrc

    remove_monitoring_for_mirror0
    
    run_plugins 0 delete
        
    # TRANSLATORS: The "domain" specified in the message is the high availability domain, which 
    # is just the container that groups all the hardware and software resources that 
    # High Availability controls
    sm_msg "`eval_gettext "Removing the domain."`"
    delete_domain
                
    #run_activation_plugins unconfigcleanup
    
    sm_msg "`eval_gettext "The high availability setup was deleted."`"
        
}

crt_tsa_model () {
    
    # the configinit step of the controlled plugins must be called prior to creating the TSA model
    # Creating the model results in the mirrors being unmounted and the plugin may need data in
    # the mirrors.

    run_controlled_plugins configinit

    # Do the setup
    # TRANSLATORS: The "domain" specified in the message is the high availability domain, which 
    # is just the container that groups all the hardware and software resources that 
    # High Availability controls
    sm_msg "`eval_gettext "Creating the domain."`"

    # always create the domain with IPv6 support. Later, when IBM.SeviceIP resources are created,
    # IPv6 support is turned off if there are no global IPv6 static addresses harvested by RSCT.

    debug "calling mkrpdomain -6 $TSA_DOMAIN $HOST $node"
    mkrpdomresult=`LC_ALL=C mkrpdomain -6 $TSA_DOMAIN $HOST $node 2>&1`
    local mkrpRC=$?
    debug "mkrpdomain rc:$mkrpRC: $mkrpdomresult"
    
    # Wait for a little bit so it can get propagated to the nodes
    sleep 5
    
    # TRANSLATORS: The "domain" specified in the message is the high availability domain, which 
    # is just the container that groups all the hardware and software resources that 
    # High Availability controls
    sm_msg "`eval_gettext "Starting the domain."`"
    record_cfg_status "Starting the domain."
    
    start_n_wait_for_domain
    
    # Turn on automation, so things come online
    debug "Turning on automation"                 
    #result=`samctrl -M F -r $TSA_RETRY_COUNT 2>&1` 
    debug "tsa call hook 10"
    result=$(call_tsa_samctrl_M_r F $TSA_RETRY_COUNT)
    local sam_rc=$?        
    debug "samctrl result:$result"
    if [ $sam_rc -ne 0 ]; then
        fatal_error "`eval_gettext \"Could not turn on TSA automation. rc:\\\$sam_rc, high availability configuration failed.  Refer to the high availability troubleshooting documentation or contact Software Support.\"`"
    fi
           
    # Touch the config file so the monitors start reporting status
    touch $SMHA_CFG_FILE
                                    
    ### Create the model ###
    
    # TRANSLATORS: The "resources" specified in the message refer to hardware and
    # software components such as disks, memory, software programs such as databases,
    # ftp servers, CIM servers, etc.  These resources are being created in the
    # high availability model.
    sm_msg "`eval_gettext "Creating the resources."`"
    
    run_plugins 0 create
    
    # DHCP Must be done when the storage is online 
    # on both nodes.  We may need to copy off the config   
    # Model DHCP if it's on
    record_dhcp_autostart
    crt_dhcp_rsrc_if_on 
    
    # Always turn off DHCP
    remove_dhcp_autostart
    
    # TRANSLATORS: The "resources" specified in the message refer to hardware and
    # software components such as disks, memory, software programs such as databases,
    # ftp servers, CIM servers, etc.  These resources are being created in the
    # high availability model.
    sm_msg "`eval_gettext "Creating storage resources."`"
    record_cfg_status "Creating storage resources."
    crt_all_drbd_rsrcs
        
    run_plugins 2 create
            
    setup_monitoring_for_mirror0 0        # the argument of 0 indicates to set up monitoring on both nodes
    local setup_rc=$?
    if [ $setup_rc -ne 0 ]; then
        fatal_error "`eval_gettext "Cannot initialize the mirrored storage monitor, high availability configuration failed.  Refer to the high availability troubleshooting documentation or contact Software Support."`"
    fi
            
    # Create the DB2 resource in TSA
    # TRANSLATORS: The "resources" specified in the message refer to hardware and
    # software components such as disks, memory, software programs such as databases,
    # ftp servers, CIM servers, etc.  These resources are being created in the
    # high availability model.
    sm_msg "`eval_gettext "Creating database resources."`"
    record_cfg_status "Creating database resources."
    crt_db2_rsrc

    # Create the DB2 dependencies        
    crt_db2_dep 

    run_plugins 4 create
    
    # TRANSLATORS: The "resources" specified in the message refer to hardware and
    # software components such as disks, memory, software programs such as databases,
    # ftp servers, CIM servers, etc.  These resources are being created in the
    # high availability model.
    sm_msg "`eval_gettext "Creating CIM resources."`"
    record_cfg_status "Creating CIM resources."
    crt_cim_rsrcs

    # Make sure director is not setup to autostart
    remove_director_autostart
    
    # Create the Director resource (OEMB)
    # TRANSLATORS: The "resources" specified in the message refer to hardware and
    # software components such as disks, memory, software programs such as databases,
    # ftp servers, CIM servers, etc.  These resources are being created in the
    # high availability model.
    sm_msg "`eval_gettext "Creating management server resources."`"
    record_cfg_status "Creating management server resources."
    crt_dir_rsrc
                                    
    # Create Director dependencies
    crt_dir_dep

    # Create floating IPs
    # TRANSLATORS: The "resources" specified in the message refer to hardware and
    # software components such as disks, memory, software programs such as databases,
    # ftp servers, CIM servers, etc.  These resources are being created in the
    # high availability model.
    sm_msg "`eval_gettext "Creating networking resources."`"
    record_cfg_status "Creating networking resources."
    crt_network_aliases
    
    run_plugins 6 create

    # Make sure ntp is not setup to autostart
    #  --- doesn't matter since TSA will just insure it's running.
#    remove_ntp_autostart

    # Create the NTP resource
    # TRANSLATORS: The "resources" specified in the message refer to hardware and
    # software components such as disks, memory, software programs such as databases,
    # ftp servers, CIM servers, etc.  These resources are being created in the
    # high availability model.
    sm_msg "`eval_gettext "Creating NTP resources."`"
    record_cfg_status "Creating NTP resources."
    crt_ntp_rsrc
    
    # Create the Quorum Tie Breaker
    if [ -n "$network_quorum" ]; then
        debug "Create Network Quorum from: $network_quorum"
        crt_network_quorum $network_quorum
    else
        crt_yes_quorum
    fi

    run_controlled_plugins create
    run_activation_plugins configinit
    run_activation_plugins activate

    # Start the base drbd rg
    # TRANSLATORS: The "resources" specified in the message refer to hardware and
    # software components such as disks, memory, software programs such as databases,
    # ftp servers, CIM servers, etc.  These resources are being created in the
    # high availability model.
    sm_msg "`eval_gettext "Starting storage resources."`"
    record_cfg_status "Starting storage resources."
    start_n_wait_for_rg_online $DRBD_RG
                                        
    # We need the storage online 
    ch_rg_online $STORAGE_RG
    
    if [ $DRBD_EQU -eq 0 ]; then
	    # Wait while drbd is syncing. It takes a while to start syncing.
	    sleep 240    
	    wait_while_syncing
    fi
    
    wait_for_rg_online $STORAGE_RG
    
    # Do the second part of the dhcp setup
    crt_dhcp_rsrc_step2
    
    # Update the agent files with the agent address
    # TRANSLATORS: The "Agent Manager" specified in the message refers to the 
    # code that runs in the IBM Director Server that manager CAS agents
    sm_msg "`eval_gettext "Configuring Agent Manager."`"
    record_cfg_status "Configuring Agent Manager."
    updt_all_agntmgr_files_primary
            
}

delete_domain() {
    
    # The domain must be online in order to delete it.
    
    debug "Start the domain"
    start_n_wait_for_domain
            
    take_all_rg_offline
    sleep 5
    wait_all_rg_offline
    
    rm_all_rg
    
    debug "Remove the domain"
    remove_n_wait_for_domain    
    
    rc=$?
    if [ $rc -ne 0 ]; then
        fatal_error "`eval_gettext \"Timed out waiting for TSA domain to be deleted. rc=\\\$rc\"`"
    fi
    
}

