# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#

#SMHA_PLUGIN_MODEL_DIR_BASE=/opt/ibm/director/ha/bin/tsa/m.
SMHA_PLUGIN_MODEL_DIR_BASE=/opt/ibm/director/ha/plugins/tsa/m.

#
# Parameters:
#   level:  The mirror level to invoke. (like, 0,1,2,3,etc)
#   method: The method to run. Either "create", or "delete"
#           The method may also be a multi-word string, to be passed as
#           individual words to the plugin script
#
run_plugins () {
    local level=$1
    local method="$2"          # quote the arg to keep as a single string

    # Export variables needed by the plugin
    export node

    local srchdir=${SMHA_PLUGIN_MODEL_DIR_BASE}${level}

    files=`ls $srchdir`
	if [ -n "$files" ]; then
        for f in $files
        do
            if [ -x $srchdir/$f ]; then
                debug "Calling: $f $method"
                $srchdir/$f $method             # no quotes to pass multiple args
            fi
        done
	
	fi

}

#SMHA_PLUGIN_DRBD_DIR_BASE=/opt/ibm/director/ha/bin/drbd/
SMHA_PLUGIN_DRBD_DIR_BASE=/opt/ibm/director/ha/plugins/drbd/

#
# Parameters:
#   method: The method to run. Either "stop", "start", "bindmounts"
#           Which means to stop or start any services that may hold open files on the resource
#           bindmount = means the next step will be to process bind mounts for this resource.
#   resource: The mount point
#
run_drbd_plugins () {
    local method=${1}
    local mountpt=${2}

    # Export variables needed by the plugin
    export node

    local srchdir=${SMHA_PLUGIN_DRBD_DIR_BASE}

    files=`ls $srchdir`
    if [ -n "$files" ]; then
        for f in $files
        do
            if [ -x $srchdir/$f ]; then
                debug "Calling: $f $method $mountpt"
                $srchdir/$f $method $mountpt 
                debug "Returning from call to: $f $method $mountpt"
            fi
        done
    
    fi

}

SMHA_PLUGIN_CONF_DIR_BASE=/opt/ibm/director/ha/plugins/conf/

#
# Parameters:
#   operation: Either "conf" or "unconf"
#   state: Either "begin", or "end"
#
run_conf_plugins () {
    local operation=${1}
    local state=${2}

    # Export variables needed by the plugin
    export node

    local srchdir=${SMHA_PLUGIN_CONF_DIR_BASE}

    files=`ls $srchdir`
    if [ -n "$files" ]; then
        for f in $files
        do
            if [ -x $srchdir/$f ]; then
                debug "Calling: $f $operation $state"
                $srchdir/$f $operation $state 
            fi
        done
    
    fi

}

SMHA_PLUGIN_ACTIVATIONS_DIR_BASE=/opt/ibm/director/ha/plugins/tsa/activations

run_activation_plugins () {
    local operation=${1}
    local flag

    # The activation list, if not NULL, is a comma separated list of
    # activation "keys", where each key may have an optional flag appended
    #
    #     NETWORK_CONTROL:1,STORAGE_CONTRL:0
    # or
    #     NETWORK_CONTROL,STORAGE_CONTROL
    #
    # Either all elements of the list have the flag or none do.
    # A flag value of 1 or NULL indicates the advanced manager is activated.
    # A flag value of 0 indicates the advanced manager is not activated.
    
    debug "In run_activation_plugins().  activations=$activations"
    local srchdir=${SMHA_PLUGIN_ACTIVATIONS_DIR_BASE}
    
    if [ -n "$activations" ]; then
        activationlist=${activations//,/ }
        for a in $activationlist
        do
            flag=${a#*:}     # get flag, if present
            a=${a%:*}        # remove flag, if present
            if [ "$a" = "$flag" ]; then
                # flag is NULL, which means "activated"
                flag=1
            fi

            if [ -x $srchdir/$a ]; then

                # if flag is 1 OR operation is configinit
                if [ $flag -eq 1 -o "$operation" = "configinit" ]; then
                    debug "Calling: $a $operation"
                    $srchdir/$a $operation
                else
                    debug "Skipping $operation for $a"
                fi
            fi
        done
    fi

}


SMHA_PLUGIN_CONTROLLED_DIR_BASE=/opt/ibm/director/ha/plugins/tsa/controlled

#
# Parameters:
#   operation: One of configinit, create, delete, unconfigcleanup
#   RGnames:   optional resource group name
#
# If one or more resource group names are specified, only the matching
# Controlled plug-in scripts are invoked

run_controlled_plugins () {
    local RC=0
    local operation=${1}
    shift

    debug "In run_controlled_plugins().  operation=$operation RGnames=$*"    # remove after unit test
    local srchdir=${SMHA_PLUGIN_CONTROLLED_DIR_BASE}

    if [ -n "$1" ]; then
        files="$*"
    else
        files=`ls $srchdir`
    fi

    if [ -n "$files" ]; then
        for f in $files
        do
            if [ -f $srchdir/$f -a -x $srchdir/$f ]; then
                debug "Calling controlled plug-in: $f $operation"
                $srchdir/$f $operation || (( RC++ ))
            else
                debug "Controlled plug-in $f is not executable or not found"
                (( RC++ ))
            fi
        done
    fi

    return $RC
}
