# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996 
#

SMHANET_QUORUM=smhanetq

crt_network_quorum () {
    local net_addr=${1}
    local c_out
    local rc
    local tb_name=$NET_QUORUM
    
    debug "Network quorum address:$net_addr"

    local six=
    is_address_ipv6 $net_addr    
    if [ $? -eq 1 ]; then
    	debug "Network quorum is IPv6 address:$net_addr"
    	six=6
    fi

    mkrsrc IBM.TieBreaker Type="EXEC" Name="$NET_QUORUM" \
    DeviceInfo="PATHNAME=/usr/sbin/rsct/bin/samtb_net${six} Address=$net_addr Log=1" \
    PostReserveWaitTime=30;

    if [ "$six" = "6" ]; then
        mkrsrc IBM.TieBreaker Type="EXEC" Name="$SMHANET_QUORUM" \
        DeviceInfo="PATHNAME=${SMHA_SCRIPTS}/priv/smhanetq.sh Address=$net_addr" \
        PostReserveWaitTime=30;

        tb_name=$SMHANET_QUORUM
    fi
    
    c_out=$(chrsrc -c IBM.PeerNode OpQuorumTieBreaker="$tb_name" 2>&1)
    rc=$?
    if [ $rc -ne 0 ]; then
        debug "Could not set OpQuorumTieBreaker rc=$rc chrsrc out: $c_out"
        exit 1     # this will spring the exit trap in smhainit.sh
    fi
    
}


crt_yes_quorum () {
    local c_out
    local rc

    debug "Create Yes quorum"
        
    mkrsrc IBM.TieBreaker Type="EXEC" Name="$Y_QUORUM" \
    DeviceInfo="PATHNAME=$SMHA_SCRIPTS/tsa/smhayquorum.sh" \
    PostReserveWaitTime=30;
    
    c_out=$(chrsrc -c IBM.PeerNode OpQuorumTieBreaker="$Y_QUORUM" 2>&1)
    rc=$?
    if [ $rc -ne 0 ]; then
        debug "Could not set OpQuorumTieBreaker rc=$rc chrsrc out: $c_out"
        exit 1     # this will spring the exit trap in smhainit.sh
    fi
    
}

crt_ntp_rsrc () {
    # Only create the resource if it doesn't already exist
    lsrg -A d -g $NTP_RG &>/dev/null
    if [ $? -eq 6 ]; then
        debug "Create the ntp resources"
        mkrsrc IBM.Application \
        Name="$NTP_RSRC_NAME" \
        StartCommand="$NTP_MONITOR_SCRIPT start" \
        StopCommand="$NTP_MONITOR_SCRIPT stop" \
        MonitorCommand="$NTP_MONITOR_SCRIPT status" \
        MonitorCommandPeriod=$NTP_MON_CMD_PERIOD \
        MonitorCommandTimeout=$NTP_MON_CMD_TIMEOUT \
        NodeNameList="{'$HOST','$node'}" \
        StartCommandTimeout=$NTP_MON_START_TIMEOUT \
        StopCommandTimeout=$NTP_MON_STOP_TIMEOUT \
        UserName="root" \
        ResourceType=2
        
        # create the resource group
        mkrg $NTP_RG
        chrg_l_none $NTP_RG
        add_to_rg $NTP_RSRC_NAME IBM.Application $NTP_RG 
        sleep 5
        #chrg -o online $NTP_RG
        debug "tsa call hook 11"
        local tsa_return
        tsa_return=$(call_tsa_chrg_o_l_u -o online $NTP_RG)
    else
        debug "ntp resources already created"   
    fi  
}

remove_ntp_autostart () {

    debug "Remove NTP autostart"

    /sbin/chkconfig -d ntp &>/dev/null
}

crt_net_alias () {

    # For each network alias, create it.
    local i
    for i in {1..5}
    do
        # See if there is an alias variable with this number
        local net_alias=network_alias__${i}
        # Dereference the constructed variable name
        local net_alias_value=${!net_alias}
        if [ -n "$net_alias_value" ]; then
            debug "Processing net alias:${net_alias_value}"
            parse_network_alias $net_alias_value
  
            # Director depends on or is collocated with this network alias
            local relationship="DependsOn"
            local mandatory_flag="T"
            # If this address is not the agent manager address then it's just collocated
            if [ "$agent_manager_address" != "$ALIAS_ADDR" ]; then
                relationship="Collocated"
                mandatory_flag="F"           # make the resource group member non-mandatory if not agent manager address
            fi
            
            # Create the equivalency between nics
            lsequ -e ${ALIAS_NIC}-equ &>/dev/null
            if [ $? -eq 6 ]; then
                debug "Make the equivalence for ${ALIAS_NIC}-equ"
                mkequ -D "Name like '${ALIAS_NIC}' && IPVersion=4" ${ALIAS_NIC}-equ IBM.NetworkInterface
            fi
            
            debug "Host: $HOST and node: $node"
            # Create the floating IP
            debug "Make the IBM.ServiceIP:${ALIAS_RSRC_NAME}${i}IP"    
            mkrsrc IBM.ServiceIP \
            Name="${ALIAS_RSRC_NAME}${i}IP" \
            NetMask=$ALIAS_MASK \
            IPAddress=$ALIAS_ADDR \
            NodeNameList="{'$HOST','$node'}"
            
            # Add this service IP to the Director RG
            add_to_rg  ${ALIAS_RSRC_NAME}${i}IP IBM.ServiceIP $DIRECTOR_RG $ADDRGMBR_ORDER $mandatory_flag

            # The alias depends on the network equivalence
            debug "Make the relationship between ServiceIP and Equivalence:${ALIAS_RSRC_NAME}${i}"            
            make_relationship DependsOn IBM.ServiceIP:${ALIAS_RSRC_NAME}${i}IP \
            IBM.Equivalency:${ALIAS_NIC}-equ \
            ${ALIAS_RSRC_NAME}${i}IP-depon-${ALIAS_NIC}-equ        
        
            debug "Make the relationship between Director and ServiceIP:${ALIAS_RSRC_NAME}${i}"
            make_relationship $relationship IBM.Application:$DIRECTOR_RSRC_NAME \
            IBM.ServiceIP:${ALIAS_RSRC_NAME}${i}IP \
            $DIRECTOR_RSRC_NAME-depon-${ALIAS_RSRC_NAME}${i}
            
            debug "Done processing net alias:${net_alias_value}"
            
        else
          # No more alias were found
          return                    
        fi
    
    done
        
} 

crt_net_alias6 () {
    local j
    local equName
    local ipversion

    # For each network alias, create it.
    local i
    for i in {1..5}
    do
        # See if there is an alias variable with this number
        local net_alias=network_alias6__${i}
        # Dereference the constructed variable name
        local net_alias_value=${!net_alias}
        if [ -n "$net_alias_value" ]; then
            debug "Processing net alias6:${net_alias_value}"
            parse_network_alias6 $net_alias_value

            # Director depends on or is collocated with this network alias
            local relationship="DependsOn"
            local mandatory_flag="T"
            # If this address is not the agent manager address then it's just collocated
            if [ "$agent_manager_address" != "$ALIAS_ADDR" ]; then
                relationship="Collocated"
                mandatory_flag="F"           # make the resource group member non-mandatory if not agent manager address
            fi

            # attempt to make an equivalency for IPv6. If there are no IPv6 addresses on the specified adapter,
            # then make an equivalency for IPv4 (if not already made)
            #
            # N.B. similar logic could be used in crt_net_alias, if there is ever need to support floating IPv4
            # addresses on adapters with no static IPv4 addresses assigned.

            j=${ALIAS_NIC#eth}
            if (( ( 1 << j ) & SMHA_NIC_WITH_IPV6 )); then
                equName=${ALIAS_NIC}v6-equ
                ipversion=6
            else
                equName=${ALIAS_NIC}-equ
                ipversion=4
            fi
            
            # Create the equivalency between nics
            lsequ -e $equName &>/dev/null
            if [ $? -eq 6 ]; then            
                debug "Make the equivalence:${equName}"
                mkequ -D "Name like '${ALIAS_NIC}' && IPVersion=${ipversion}" ${equName} IBM.NetworkInterface
            fi
            
            debug "Host: $HOST and node: $node"
            # Create the floating IP
            debug "Make the IBM.ServiceIP:${ALIAS6_RSRC_NAME}${i}IP"
            # The network prefix is optional...see if it's specified.
            if [ -n "$ALIAS_PREFIX" ]; then
                mkrsrc IBM.ServiceIP \
                Name="${ALIAS6_RSRC_NAME}${i}IP" \
                NetPrefix=$ALIAS_PREFIX \
                IPAddress=$ALIAS_ADDR \
                NodeNameList="{'$HOST','$node'}"
            else
                mkrsrc IBM.ServiceIP \
                Name="${ALIAS6_RSRC_NAME}${i}IP" \
                IPAddress=$ALIAS_ADDR \
                NodeNameList="{'$HOST','$node'}"
            fi
            
            # Add this service IP to the Director RG
            add_to_rg ${ALIAS6_RSRC_NAME}${i}IP IBM.ServiceIP $DIRECTOR_RG $ADDRGMBR_ORDER $mandatory_flag

            # The alias depends on the network equivalence
            debug "Make the relationship between ServiceIP and Equivalence:${ALIAS6_RSRC_NAME}${i} ${equName}"
            make_relationship DependsOn IBM.ServiceIP:${ALIAS6_RSRC_NAME}${i}IP \
            IBM.Equivalency:${equName} \
            ${ALIAS6_RSRC_NAME}${i}IP-depon-${equName}
        
            debug "Make the relationship between Director and ServiceIP:${ALIAS6_RSRC_NAME}${i}"
            make_relationship $relationship IBM.Application:$DIRECTOR_RSRC_NAME \
            IBM.ServiceIP:${ALIAS6_RSRC_NAME}${i}IP \
            $DIRECTOR_RSRC_NAME-depon-${ALIAS6_RSRC_NAME}${i}
            
            debug "Done processing net alias:${net_alias_value}"
            
        else
          # No more alias were found
          return                    
        fi
    
    done
        
} 

# Check what IPv6 global addresses have been harvested by RSCT. If there are no global
# addresses then disable IPv6 support in RSCT. Also check which IPv4 addresses have been
# harvested by RSCT.
#
# As a result of these checks, the variables SMHA_NIC_WITH_IPV6 and SMHA_NIX_WITH_IPV4 are set.
# These values are bit masks, with the least significant bit corresponding to eth0, next bit to eth1, etc.
# A bit that is set indicates that the corresponding interface has that type of IP address on both nodes.
# These masks are used to determine which equivalencies of IBM.NetworkInterface resources can be created.

check_rsct_ip () {
    local i
    local A=0
    local B=0
    local token
    local ipaddr
    local name
    local chrsrc_out
    local rc

    get_local_rsct_node_name
    get_remote_rsct_node_name

    SSPFX="IPVersion == 6 && !(IPAddress LIKE 'fe80%' || IPAddress LIKE 'FE80%') && NodeNameList |<"
    for token in $(LC_ALL=C lsrsrc-api -D '|' -s IBM.NetworkInterface::"${SSPFX} {'$LOCAL_RSCT_NODE_NAME'}"::Name::IPAddress 2> /dev/null); do
        name=${token%|*}
        ipaddr=${token#*|}
        debug "check_rsct_ip: $LOCAL_RSCT_NODE_NAME name=$name  ipaddr=$ipaddr"

        i=${name#eth}
        (( A |= 1 << i ))
    done

    for token in $(LC_ALL=C lsrsrc-api -D '|' -s IBM.NetworkInterface::"${SSPFX} {'$REMOTE_RSCT_NODE_NAME'}"::Name::IPAddress 2> /dev/null); do
        name=${token%|*}
        ipaddr=${token#*|}
        debug "check_rsct_ip: $REMOTE_RSCT_NODE_NAME name=$name  ipaddr=$ipaddr"

        i=${name#eth}
        (( B |= 1 << i ))
    done

    if (( A == 0 || A != B )); then
        # Either there are no IPv6 addresses or the interfaces with IPv6 are
        # not consistent on the two nodes. Turn off IPv6

        chrsrc_out=`chrsrc -c IBM.NetworkInterface IPv6Support=0`
        rc=$?
        debug "check_ipv6 set IPv6Support=0: rc=$rc : output is: $chrsrc_out"
    else
        run_init "update-ipv6-addresses" $HOST $node
        rc=$?
        debug "check_ipv6 called run_init update-ipv6-addresses: rc=$rc"

        # kick off a refresh of the IBM.NetworkInterface resources
        refrsrc IBM.NetworkInterface
        # This is supposed to complete within about 30 seconds
        sleep 60
    fi

    SMHA_NIC_WITH_IPV6=$A

    A=0
    B=0
    SSPFX="IPVersion == 4 && NodeNameList |<"
    for token in $(LC_ALL=C lsrsrc-api -D '|' -s IBM.NetworkInterface::"${SSPFX} {'$LOCAL_RSCT_NODE_NAME'}"::Name::IPAddress 2> /dev/null); do
        name=${token%|*}
        ipaddr=${token#*|}
        debug "check_rsct_ip: $LOCAL_RSCT_NODE_NAME name=$name  ipaddr=$ipaddr"

        i=${name#eth}
        (( A |= 1 << i ))
    done

    for token in $(LC_ALL=C lsrsrc-api -D '|' -s IBM.NetworkInterface::"${SSPFX} {'$REMOTE_RSCT_NODE_NAME'}"::Name::IPAddress 2> /dev/null); do
        name=${token%|*}
        ipaddr=${token#*|}
        debug "check_rsct_ip: $REMOTE_RSCT_NODE_NAME name=$name  ipaddr=$ipaddr"

        i=${name#eth}
        (( B |= 1 << i ))
    done

    if (( A == 0 || A != B )); then
        debug "mismatch in IPv4 addresses"
        A=$(( A & B ))
        if (( A == 0 )); then
            debug "no IPv4 addresses available for equivalencies"
        fi
    fi

    SMHA_NIC_WITH_IPV4=$A

    debug "SMHA_NIC_WITH_IPV4=$SMHA_NIC_WITH_IPV4  SMHA_NIC_WITH_IPV6=$SMHA_NIC_WITH_IPV6"
}

crt_network_aliases () {
 	# check the IP addresses harvested by RSCT
    check_rsct_ip
    crt_net_alias
    crt_net_alias6
}
