#!/bin/bash
#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2011, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

#
# 09N6996
#

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

PARM_FLAGS="h help"
PARM_MULTI=""
PARM_TRANSLATION="h help"
PARM_SINGLES="help"
PARM_VALID=$PARM_TRANSLATION

load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh

# the following function looks for any fixed (constituent) resource that is either failed offline (3) or
# stuck online (4) in the resource class specified by the function argument

check_for_failed_off_stuck_on () {
    local RC=0
    local RsrcName

    RsrcName=$1

    #################
    # NOTA BENE !!  The following loop actually ends up executing as a sub-shell.  This means that you can't "return" a return code.
    # To work around that, we use ()'s to create another subshell, with the while statement following by an exit statement to return
    # the exit code back to the invoking shell.
    #################

    export IFS='|'
    lsrsrc-api -D '|' -s "${RsrcName}::ResourceType == 0::Name::OpState::NodeNameList" | \
    (
        local aRC=0

        while read Name OpState NodeNameList
        do
            # Check for failed offline or stuck online
            if [ $OpState -eq 3 -o $OpState -eq 4 ]
            then
                # strip {} from NodeNameList value
                NodeNameList=${NodeNameList#\{}
                NodeNameList=${NodeNameList%\}}

                echo "$Name $NodeNameList $OpState"
                aRC=1
            fi
        done
        exit $aRC
    )
    RC=$?
    return $RC
}

parse_all_parms $*
if [ -n "$help" ]
then
    echo "smhachkreset.sh [ -h | --help ]"
    echo " -h | --help   show this help"
    echo " With no options, list any fixed resources that require reset"
    exit 0
fi

check_for_failed_off_stuck_on IBM.Application
RC1=$?

check_for_failed_off_stuck_on IBM.ServiceIP
RC2=$?

RC=$(( RC1 | RC2 ))

exit $RC
