#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

export PATH=/opt/ibm/director/bin:/opt/ibm/director/lwi/bin:/bin:/usr/bin:/usr/sbin:/sbin

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi

LCL_DM_FILE=smhadm.conf
LCL_KEYSTORE=${SMHA_CONFIGDIR}/broker.ks


#################################################
#  Main code
#################################################
#debug "parameters:$*"
while getopts m:c:k:u:O:A:S: flag;do
#	debug "flag:$flag"
    case $flag in
        m) AM_PASSWD=$OPTARG ;;
        c) RMT_CONF_FILE_PATH=$OPTARG ;;
        k) RMT_KEYSTORE_FILE=$OPTARG ;;
        u) CFG_USER=$OPTARG ;;
        A) activations=$OPTARG ;;
        S) secondary_node_cas_ip=$OPTARG ;;

    esac
done

# Remove files indicating previous HA config error (if they exist)
rm -rf $ERROR_DURING_CONFIG_FLAGFILE
rm -rf $CFG_STATUS_FILE
rm -rf $DM_CONF
rm -rf $BROKER_KS

if [ -n "${RMT_KEYSTORE_FILE}" ]; then
  
  if [ -f "${RMT_KEYSTORE_FILE}" ]; then
    # move the keystore over
    mv ${RMT_KEYSTORE_FILE} ${LCL_KEYSTORE}
    chmod 400 ${LCL_KEYSTORE}
    chown root:root ${LCL_KEYSTORE}
    # Remove any MQ data
    rm -rf $SMHA_DIR/persist/*       
    # if the mq server is running it needs to be restarted
    /etc/rc.d/smhamq restart
    wait_for_mq_online
  fi

fi


if [ -n "${RMT_CONF_FILE_PATH}" ]; then

  if [ -f "${RMT_CONF_FILE_PATH}" ]; then
    # Copy the conf file over and start up the daemon
    if [ -e "${SMHA_CONFIGDIR}/$LCL_DM_FILE" ]; then
        rm -rf ${SMHA_CONFIGDIR}/$LCL_DM_FILE
    fi
    mv ${RMT_CONF_FILE_PATH} ${SMHA_CONFIGDIR}/$LCL_DM_FILE
  
    /etc/rc.d/smhadm restart
    wait_for_daemon_online
  fi

fi

if [ -n "$CFG_USER" ]; then
    save_cfg_user_id $CFG_USER
fi

# Do initial config for this node
if [ -n "${activations}" ]; then
    debug "Calling smhasecinit.sh --setup -S ${secondary_node_cas_ip} -A ${activations}"
    $INSTALL_DIR/ha/bin/priv/smhasecinit.sh --setup -S ${secondary_node_cas_ip} --password=${AM_PASSWD} -A ${activations}
else
    debug "Calling smhasecinit.sh --setup -S ${secondary_node_cas_ip}"
    $INSTALL_DIR/ha/bin/priv/smhasecinit.sh --setup -S ${secondary_node_cas_ip} --password=${AM_PASSWD}
fi

