#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
PATH=$PATH:/usr/smrshbin/
INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh
ACTIVATION_PATH=/opt/ibm/director/ha/plugins/tsa/activations

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
## Run termination processing for HA
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadrbd.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadb2.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadhcp.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadir.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaerrcode.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaplugin.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaagent.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhacim.sh

PARM_FLAGS="F force F force r reset R restore z discard-floating-addrs"
PARM_MULTI=""
PARM_TRANSLATION="F force r reset R restore A activations z discard-floating-addrs"
PARM_SINGLES=""
PARM_VALID=$PARM_TRANSLATION


###################################
## Functions                     ##
###################################
proc_parms () {
    
    # Parse the parameters
    # The result should be that variables are set for each parameter and
    # CMD_PARMS will contain the list of parameters processed
    # If a file is specified it will also read the file.
    parse_all_parms $*
    
    # this is only for debug
    #list_parm_vars  
    
    
}

ensure_domain_is_deleted () {


    if [ -z "$force" ]; then

    	local i
    	for i in {1..180}
        do
            lsrpdomain -x -l $TSA_DOMAIN &> /dev/null
            if [ $? -eq 6 ]; then
            	return
        	else
                debug "Domain not deleted.  Wait."
                sleep 10
            fi
        done
    else
        remove_n_wait_for_domain 
    fi    	

}

replaceIPAddresses()
{
    local adapter=$1
    local newIPv4Address=$2
    local newIPv6AddressList=$3
    local finalAddressList=

    if [ ! -n "$newIPv4Address" ]; then
        if [ ! -n "$newIPv6AddressList" ]; then
            return 0
        fi
    fi

    if [ -n "$newIPv4Address" ]; then
        finalAddressList=${newIPv4Address},
    else
        local existingIPv4Address=`/usr/smrshbin/lsnetcfg -n -F ipv4addr_$adapter`
        if [ "$existingIPv4Address" != "0.0.0.0" ]; then
            local existingIPv4Mask=`/usr/smrshbin/lsnetcfg -n -F ipv4netmask_$adapter`
            finalAddressList=${existingIPv4Address}/${existingIPv4Mask},
        fi
    fi

    # Need to take the autoconfigure addresses out (if any), since we don't set those
    local existingIPv6Addresses=`/usr/smrshbin/lsnetcfg -n -F ipv6addr_$adapter`

    # loop through IPv6 addresses once to find link local address (if one exists) so
    # we can find the autoconfigure part
    local autoconfigurePart=
    for addr in $(echo $existingIPv6Addresses | tr "," "\n")
    do
        addrOnly=${addr%/*}
        isLinkLocalAddr $addrOnly
        isLLAddrRC=$?
        if [ isLLAddrRC ]; then
            expandIPv6Address $addrOnly
            autoconfigurePart=${RETURN_EXPANDED_ADDR#*:*:*:*:}
        fi
    done

    # loop through again, and remove the addresses that have the autoconfigure part,
    # both the link local and public autoconfigure addresses
    for addr in $(echo $existingIPv6Addresses | tr "," "\n")
    do
        addrOnly=${addr%/*}
        expandIPv6Address $addrOnly
        lastPart=${RETURN_EXPANDED_ADDR#*:*:*:*:}
        if [ "$lastPart" != "$autoconfigurePart" ]; then
            finalAddressList=${finalAddressList}${addr},
        fi
    done

    # Now add the new IPv6 addresses to the finalAddressList
    for addr in $(echo $newIPv6AddressList | tr "," "\n")
    do
        finalAddressList=${finalAddressList}${addr},
    done

    # Get rid of the ending , on finalAddressList
    if [ -n "$finalAddressList" ]; then
        finalAddressList=${finalAddressList%,}
    fi

    debug "About to run the following command to replace floating addresses: /usr/smrshbin/chnetcfg -s modify -i $adapter -a $finalAddressList"
    /usr/smrshbin/chnetcfg -s modify -i $adapter -a $finalAddressList
}


keep_floating_addresses ()
{
    oldHostnameAndIPPropValue=$1
    shift
    oldShortname=
    oldHostIPAddr=

    if [ "$oldHostnameAndIPPropValue" != "NONE" ]; then
        oldHostIPAddr=${oldHostnameAndIPPropValue#*,}
        oldShortname=${oldHostnameAndIPPropValue%,*}
        debug "old-hostname value $oldHostnameAndIPPropValue split into $oldShortname and $oldHostIPAddr"
    fi

    # rest of parameters are of form "adapter:floatingAddrs"
    while (( "$#" )); do
        local parmToProcess=$1
        shift
        debug "processing $parmToProcess"
        
        local adapterName=${parmToProcess:0:4}
        local floatingaddresses=${parmToProcess:5}
        debug "adapterName=${adapterName},floatingaddresses=${floatingaddresses}"
        local ipv4FloatingAddr=
        local ipv6FloatingAddrList=
        local netalias=

        for netalias in ${floatingaddresses}
        do
            local isIPv6=`echo $netalias | grep alias6`
            local value=${netalias##*=}
            local addressOnly=${value%%,*}
            local addressAndMask=${value%,*}
            addressAndMask=${addressAndMask/,/\/}
            #echo "value=$value   addressOnly=$addressOnly   addressAndMask=$addressAndMask"
            if [ -n "$isIPv6" ]; then
                if [ -n "$ipv6FloatingAddrList" ]; then
                    ipv6FloatingAddrList=$ipv6FloatingAddrList,$addressAndMask
                else
                    ipv6FloatingAddrList=$addressAndMask
                fi
            else
               if [ -n "$ipv4FloatingAddr" ]; then
                   if [ "$addressOnly" == "$agent_manager_address" ]; then
                       ipv4FloatingAddr=$addressAndMask
                   fi
               else
                   ipv4FloatingAddr=$addressAndMask
               fi
            fi
        done
        debug "${adapterName} IPv6 floating addresses to replace: $ipv6FloatingAddrList"
        debug "${adapterName} IPv4 floating address to replace: $ipv4FloatingAddr"

        if [ -n "$ipv4FloatingAddr" -o -n "ipv6FloatingAddrList" ]; then
            replaceIPAddresses "$adapterName" "$ipv4FloatingAddr" "$ipv6FloatingAddrList"
            
            if [ -n "$oldShortname" -a -n "$oldHostIPAddr" ]; then
                addrOnly=${ipv4FloatingAddr%/*}
                if [ "$oldHostIPAddr" == "$addrOnly" ]; then
                    debug "calling /usr/smrshbin/chnetcfg -s modify --host-name $oldShortname --host-ip-address $oldHostIPAddr"
                    /usr/smrshbin/chnetcfg -s modify --host-name $oldShortname --host-ip-address $oldHostIPAddr
                else
                    expandIPv6Address $oldHostIPAddr
                    expandedOldHostIPAddr=$RETURN_EXPANDED_ADDR
                
                    for addr in $(echo $existingIPv6Addresses | tr "," "\n")
                    do
                        addrOnly=${addr%/*}
                        expandIPv6Address $addrOnly
                        if [ "$RETURN_EXPANDED_ADDR" == "$expandedOldHostIPAddr" ]; then
                            debug "calling /usr/smrshbin/chnetcfg -s modify --host-name $oldShortname --host-ip-address $oldHostIPAddr"
                            /usr/smrshbin/chnetcfg -s modify --host-name $oldShortname --host-ip-address $oldHostIPAddr
                        fi
                    done
                fi
            fi
        fi
    done
}


###################################
## Main code                     ##
###################################
RC=0

proc_parms $*

debug "Running final termination code."

# Mark that termination has started
record_starting_termination

# Make sure the domain no longer exists
ensure_domain_is_deleted

comment_out_ha_inittab

# In the normal case this should already have been done
/etc/rc.d/smhamq stop
/etc/rc.d/smhadm stop

# Remove any MQ data
rm -rf $SMHA_DIR/persist/*

# Mount the mirrors
mount_mirrors

remove_dir_ha_props
dirc_use_settings active

#rm -rf /etc/ibm/tivoli/AM.guid

# load the agent manager address
get_agntmgr_addr

# Stop the agent
endpoint_dir=$INSTALL_DIR/agent/runtime/agent/bin
$endpoint_dir/endpoint.sh stop &> /dev/null

revert_cas_ip_property

# if we're on the passive node, or if we're on the active but the user chose not to keep the 
# floating addresses, then reset the agent manager address to a static one on the node
if [ -n "$reset" -o -n "$discard_floating_addrs" ]; then
    # map the floating address to a static    
    find_static_address_for_floater $agent_manager_address
    static_address=$RETURN_STATIC_ADDR
    debug "changing agent manager address to static address $static_address from $agent_manager_address"
    updt_agent_endpt_prop $static_address
    replace_agentmgr_ARShost_addr $static_address
    updt_am_mgr_prop $static_address
else
    debug "Leaving agent manager address as $agent_manager_address"
fi

if [ -n "$restore" ]; then
	# Start the CAS agent back up
	debug "starting cas agent"
	$endpoint_dir/endpoint.sh start &> /dev/null
fi

turn_on_address_change_processing
remove_force_address_change_processing_property
set_am_mnic_on

terminate_dhcpd__restore_orig_config

# Re-enable any applications
if [ -n "$reset" ]; then
	debug "not turning director autostart or other apps back on since this node needs to be reinstalled"
else
	restore_dirstartservice_status
	restore_dhcp_if_needed
	# Re-enable db2
	uncomment_fmc
	db2auto_on

	if [ -n "$activations" ]; then
	    for activation in $activations
	    do
	      debug "calling $activation unconfigcleanup"
	      $ACTIVATION_PATH/$activation unconfigcleanup
	      activation_rc=$?
	      debug "$activation unconfigcleanup rc=$activation_rc"
	    done
	fi

        run_controlled_plugins unconfigcleanup
fi

run_conf_plugins unconf end

if [ -n "$restore" -a ! -n "$discard_floating_addrs" ]; then

    oldHostnameAndIP=NONE
    get_value_from_cfg old-hostname
    if [ -n "$CFG_RETURN_VALUE" ]; then
        oldHostnameAndIP=$CFG_RETURN_VALUE
        debug "old-hostname value $oldHostnameAndIP retrieved from config file"
    fi

    eth0FloatAddrs=eth0:`grep eth0 $SMHA_CFG_FILE | grep network-alias`
    eth1FloatAddrs=eth1:`grep eth1 $SMHA_CFG_FILE | grep network-alias`
    eth2FloatAddrs=eth2:`grep eth2 $SMHA_CFG_FILE | grep network-alias`
    eth3FloatAddrs=eth3:`grep eth3 $SMHA_CFG_FILE | grep network-alias`

    # Remove the configuration files because we are unconfiguring HA, and because keep_floating_addresses call
    # will fail if the HA config files are there (because chnetcfg won't allow the commands we use because it
    # will think HA is still configured)
    debug "removing files in config directory"
    rm -rf $SMHA_DIR/config/*
    rm -rf $SMHA_DIR/config/.cfgstatus

    keep_floating_addresses "$oldHostnameAndIP" "$eth0FloatAddrs" "$eth1FloatAddrs" "$eth2FloatAddrs" "$eth3FloatAddrs"
else
    # Remove the configuration files
    debug "removing files in config directory"
    rm -rf $SMHA_DIR/config/*
    rm -rf $SMHA_DIR/config/.cfgstatus
fi

# Backup the drbd config, just in case
if [ -f /etc/drbd.conf ]; then
	if [ -f /etc/drbd.conf.bkup ]; then
		rm /etc/drbd.conf.bkup
	fi
    mv /etc/drbd.conf /etc/drbd.conf.bkup
fi

if [ -n "$reset" ]; then

#	debug "starting db2"
#	/bin/su - db2inst1 -c  'db2stop force >/dev/null 2>&1; db2start >/dev/null 2>&1'
#	db2_list=`/bin/su - db2inst1 -c "db2 list db directory"`
#	debug "db2 list:$db2_list"
	
#	debug "running smreset"
#	export PATH=$PATH:/home/db2inst1/sqllib/bin:/usr/lib/mit/bin:/usr/lib/mit/sbin:/opt/ibm/director/bin:/opt/ibm/director/tpm/tools
#    /opt/ibm/director/bin/smreset 2>&1 >$LOG_DIR/smreset.log <<<yes
#    debug "reset done"

	# instead of running smreset (since it doesn't reset agent manager data completely), we need to
	# disable director (agent and server) so that the user is forced to reinstall

	# Add a flag file indicating director isn't usable due to HA unconfigure.  /usr/smrshbin/smstart will check this flag file
	echo "Director Server, Agent Manager, and CAS Agent should not be used on this system because they are in an inconsistent state due to High Availability being unconfigured" > ${SMHA_CONFIGDIR}/.haConfigRemoved
	debug "Done with HA termination."
else
	# Don't try restarting resources, just reboot
	debug "Done with HA termination. Rebooting..."
	/usr/smrshbin/smshutdown -t now -r
	exit $RC
fi


