#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#
# 
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
#### Use a special path for TSA monitoring scripts
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/sbin/rsct/bin
export PATH

#### Standard code to find an pull in all sm common code ####
INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/tsa
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhadrbd.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaplugin.sh

DRBD_LOCK_FILE=$SMHA_CONFIGDIR/lock-
DRBD_STARTING_FILE=$SMHA_CONFIGDIR/.drbd_starting_flag
NO_MIRRORS_FILE=$SMHA_CONFIGDIR/.drbd_nomirrors_flag
SPLIT_BRAIN_FILE=$SMHA_CONFIGDIR/.drbd_splitbrain_flag

# Keep track if we stopped a resource temporarily
RESOURCE_STOPPED=

#################################################
#  Functions
#################################################
get_drbd_version ()  {
    if [ -z "$DRBD_VER" ]; then
       local kernel_version=`uname -r`
       DRBD_VER=`grep -m 1 "drbd" /lib/modules/"$kernel_version"/modules.dep | awk -F/ '{print $NF}' | cut -d'.' -f 1`
    fi
		
	debug "drbd version is:$DRBD_VER"	
}

modprobe_drbd () {
	get_drbd_version
	local mp=`modprobe $DRBD_VER 2>&1`
	debug "modprobe $DRBD_VER  output:$mp"
}

modprobe_r_drbd () {
	get_drbd_version
	
	local try_count=0
	while true; do
		local mp=`modprobe -r $DRBD_VER 2>&1`
		local mp_rc=$?
		debug "Try:$try_count modprobe -r $DRBD_VER  rc:$mp_rc output:$mp"
		if [ -z "$mp" ]; then
			break
		else
		    let "try_count=try_count + 1"
		    if [ $try_count -le 8 ]; then
		       sleep 5
		    else
		       break
	        fi
		fi
	done
	
	
}

# Ensure that we don't have something like /dev/sdf1 /opt/ibm/director/data/updateslib 
# in /proc/mounts.  If it is found mounted, it needs to be on a /dev/DRBDx device.

ensure_updateslib_not_mounted_incorrectly () {

	local updateslib_mountpt=`cat /proc/mounts | grep data/updateslib | cut -d' ' -f1`
	if [ ! -z $updateslib_mountpt ]; then
		if [[ ${updateslib_mountpt:0:9} = "/dev/drbd" ]]; then
			debug "ensure_updateslib_not_mounted_incorrectly: updateslib mount is OK on $updateslib_mountpt"
		else
			debug "ensure_udpateslib_not_mounted_incorrectly: unmounting /opt/ibm/director/data/updateslib from $updateslib_mountpt"
			cmdout=`umount /opt/ibm/director/data/updateslib`
			umount_rc=$?
			debug "ensure_updateslib_not_mounted_incorrectly: umount /opt/ibm/director/data/updateslib : rc=$umount_rc : cmd output: ${cmdout}"
		fi
	fi
}

ensure_backingstore () {
    local target=$1
    # first, let's ensure that the operating system knows that there is an error
    # by doing a list of the mount point, that forces an error even with caching
    /bin/ls $target >/dev/null 2>&1
    # now, let's write a hidden file with a timestamp to the mount point so that DRBD
    # notices an error if there is a problem
    the_date=`/bin/date +%F\ %T.%N`
    echo "$the_date" > $target/.smhatimestamp
    local rc=$?
    if [[ $rc -ne 0 ]]; then
       debug "ensure_backingstore: write of timestamp ->>$the_date<<- to $mountpt failed with rc=$rc"
    fi
}
clean_up () {
#	debug "Cleanup start"
    if [ -n "$RESOURCE_STOPPED" ]; then
       debug "Cleanup calling plugin for $RESOURCE_STOPPED"
	   run_drbd_plugins start $RESOURCE_STOPPED
    fi
    
#    debug "Cleanup done"	
}

trap "clean_up" SIGTERM EXIT 


clear_invalidate_flag () {
    local resourcename=$1
    
    rm -f $SMHA_CONFIGDIR/$DRBD_INVALIDATE_FILE\.${resourcename}
}

check_invalidate_flag () {
	local resourcename=$1 
	
	if [ -e $SMHA_CONFIGDIR/$DRBD_INVALIDATE_FILE\.${resourcename} ]; then
		debug "check_invalidate_flag: found invalidate_flag for ${resourcename}"
        # if we're already syncing then just ignore it
        is_syncing $name
        if [ $? -eq 0 ]; then
            #cmd_out=`drbdadm invalidate-remote all`
            cmd_out=`drbdadm invalidate-remote ${resourcename}`
            temprc=$?
            debug "Invalidating remote. rc: $temprc output:$cmd_out"
        fi  				
		clear_invalidate_flag $resourcename
		sleep 10
	fi
}

if_syncing_clear_flag () {
	local resourcename=$1
	
    if [ -e $SMHA_CONFIGDIR/$DRBD_INVALIDATE_FILE\.${resourcename} ]; then
        
        # if we're already syncing then just ignore it
        is_syncing $name
        if [ $? -eq 1 ]; then
            clear_invalidate_flag $resourcename
        fi
    fi
	
}

set_no_mirrors_flag () {
    touch $NO_MIRRORS_FILE
}

reset_no_mirrors_flag () {
    rm -f $NO_MIRRORS_FILE
}

set_drbd_starting_flag () {
    touch $DRBD_STARTING_FILE
}

reset_drbd_starting_flag () {
    rm -f $DRBD_STARTING_FILE
}


is_primary () {
    running=`drbdadm role $name | grep "Primary/"`
    debug "is_primary: $name $running"    
    if [ "x$running" == "x"  ]
    then
        return 0
    else
        return 1
    fi

    return 0	
}


crt_metadata () {
    local mirror=${1}
    
    debug "Create meta-data for $mirror."
    drbdadm -f create-md ${mirror} 2>&1 <<-EOF
yes
yes
EOF
    debug "create-md rc:$?"

}

crt_metadata_if_needed () {
	local mirror=${1}
	
	chk_mirror=`grep $mirror $MD_FILE`
	if [ -z "$chk_mirror" ]; then
		# We didn't find this mirror in the file
		# Create the meta data
        crt_metadata $mirror		
		echo "$mirror" >> $MD_FILE
	fi
	
}

find_pids_with_openfiles () {
	# Pass in the path of interest
    of=${1}
    LOCK_PIDS=`LC_ALL=C lsof ${1} | grep -v COMMAND | awk '{print $2}'`
    
    local lpid
    for lpid in $LOCK_PIDS
    do
    	local psStr=`ps -aefww | grep " $lpid " | grep -v grep`
        debug "Found PID $lpid with lock on:$of"
        debug "PID $lpid ps:$psStr"
    done
    
}

kill_proc_for_mirror () {
    local mirror=${1}
    local force=${2}
    
    if [ "x$force" == "xY" ]; then
    	nine="-9"
    fi

    # First find the mount points for the mirror
    find_mountpoints_for_mirror $mirror
    pts="$MIRROR_MOUNT_PTS"
    for pts in ${pts}
    do
        debug "Find pids to kill $pts"       
        find_pids_with_openfiles $pts
        local lpids=$LOCK_PIDS
#        debug "List of pids to kill:$lpids:"       
        
        local lpid
        for lpid in ${lpids}
        do
        	debug "kill $nine process $lpid"
        	kill $nine $lpid
        done         
               
    done
    
    
}

kill_smstatus () {
	debug "kill smstatus called"
	local smstatus_pid=`ps -aefww | grep plugin.USMIStatusTest | grep -v grep | awk '{print $2}'`
	debug "smstatus pid:$smstatus_pid"
	if [ -n "$smstatus_pid" ]; then
		kill $smstatus_pid
	fi
	
	
}

umount_all_mirrors () {    
    # Get the list of mirrors        
    tmpf=`mktemp /tmp/smha_uam.XXXXXX`  || fatal_error "`eval_gettext "Could not make temporary file"`"
    $GETMIRROR_CMD > $tmpf
    local mdev=
    local mmir=
    local mlab=
    while read -r line
    do
        mdev=`echo $line | cut -d' ' -f1` 
        mmir=`echo $line | cut -d' ' -f2`
        mlab=${mmir:1}
        # check if this mirror has a dependency.  If so, skip it.
       	check_for_dependent_mirror $mmir
       	if [ $? -eq 0 ]; then 
            # example: mdev=/dev/hda5, mmir=/mirror0, mlab=mirror0
            debug "umount_all_mirrors: first loop - calling umount_all $mmir $mlab"
        	umount_all $mmir $mlab
        else 
        	# skip it for now.  We will run through the list one more time after this to pick it up
        	debug "umount_all_mirrors: skipping $mmir for now - others are dependent on it."
        fi
    done < $tmpf    
    rmcmdout=`rm $tmpf`
    rmrc=$?
    debug "umount_all_mirrors: first rm cmdout= ${rmcmdout}   : rc=$rmrc"
	
	# now run through the list one more time, but this time, only process the ones that
	# we skipped the first time.
	tmpf=`mktemp /tmp/smha_uam.XXXXXX`  || fatal_error "`eval_gettext "Could not make temporary file"`"
    $GETMIRROR_CMD > $tmpf
	while read -r line
    do
        mdev=`echo $line | cut -d' ' -f1` 
        mmir=`echo $line | cut -d' ' -f2`
        mlab=${mmir:1}
    	check_for_dependent_mirror $mmir
       	if [ $? -eq 1 ]; then 
            # we skipped this one the last time through. process it.
            # example: mdev=/dev/hda5, mmir=/mirror0, mlab=mirror0
            debug "umount_all_mirrors: second loop - calling umount_all $mmir $mlab"
        	umount_all $mmir $mlab
        fi
    done < $tmpf    
    rmcmdout=`rm $tmpf`
    rmrc=$?
    debug "umount_all_mirrors: second rm cmdout= ${rmcmdout}  : rc=$rmrc"  
}

crt_all_metadata () {
    # Get the list of mirrors        
    local tmpf=`mktemp /tmp/smha_cam.XXXXXX`  || fatal_error "`eval_gettext "Could not make temporary file"`"
    $GETMIRROR_CMD > $tmpf
    while read -r line
    do
        local mdev=`echo $line | cut -d' ' -f1` 
        local mmir=`echo $line | cut -d' ' -f2`
        local mlab=${mmir:1}
        # mdev=/dev/hda5, mmir=/mirror0, mlab=mirror0
        crt_metadata_if_needed $mlab
         
    done < $tmpf    
    rm $tmpf
    
}

any_mirror_mounts () {

    # Get the list of mirrors        
    local tmpf=`mktemp /tmp/smha_amm.XXXXXX`  || fatal_error "`eval_gettext "Could not make temporary file"`"
    $GETMIRROR_CMD > $tmpf
    while read -r line
    do
        local mdev=`echo $line | cut -d' ' -f1` 
        local mmir=`echo $line | cut -d' ' -f2`
        local mlab=${mmir:1}
        # mdev=/dev/hda5, mmir=/mirror0, mlab=mirror0
        any_mounts $mlab
        if [ $? -eq 1 ]; then
            rm $tmpf
        	return 1
        fi
         
    done < $tmpf    
    rm $tmpf

    return 0    
}

find_mounts_for_mirror () {
    local mirror=${1}
    
    # the following grep will only match the bind mounts

    MIRROR_MOUNTS=`LC_ALL=C mount | grep "^/${mirror}" | cut -d' ' -f1 | grep -v spare`
    local mountout=`LC_ALL=C mount`
    debug "find_mounts_for_mirror: mount output: $mountout"
    debug "find_mounts_for_mirror:$MIRROR_MOUNTS:"

}

find_mountpoints_for_mirror () {
    local mirror=${1}
    
    MIRROR_MOUNT_PTS=`LC_ALL=C mount | grep $mirror | cut -d' ' -f3 | grep -v spare`
    debug "find_mountpoints_for_mirror:$MIRROR_MOUNTS:"

}

is_mount_on_drbd_device() {
	local mountpt=${1}
	
	local mounted_device=`LC_ALL=C cat /proc/mounts | grep $mountpt | cut -d' ' -f1`
	if [[ "$mounted_device" == /dev/drbd* ]]; then
        MOUNTED_ON_DRBD=1
	else
        MOUNTED_ON_DRBD=0
    fi
    debug "is_mount_on_drbd_device: $MOUNTED_ON_DRBD:" 
}


umount_mirror() {
    local mirror=${1}
    
    # this function is only called by umount_all, which calls any_mounts before calling this function,
    # and any_mounts calls find_mounts_for_mirror, which will have set MIRROR_MOUNTS

    # find_mounts_for_mirror $mirror

    for m in $MIRROR_MOUNTS
    do
    	debug "umount_mirror: Unmounting: $m"
    	if [[ "${m}" == "/mirror0/director/lwi/conf/overrides" ]]; then
    		debug "umount_mirror: Calling copy_directory_totemp ${DIR_OVERRIDES_DIR} ${DIR_OVERRIDES_TMP}"
            is_mount_on_drbd_device ${DIR_OVERRIDES_DIR}
    		copy_directory_totemp ${DIR_OVERRIDES_DIR} ${DIR_OVERRIDES_TMP}
    	elif [[ "${m}" == "/mirror0/director/lwi/security" ]]; then
    	    debug "umount_mirror: Calling copy_directory_totemp ${LWI_SECURITY_DIR} ${LWI_SECURITY_TMP}"
            is_mount_on_drbd_device ${LWI_SECURITY_DIR}
    	    copy_directory_totemp ${LWI_SECURITY_DIR} ${LWI_SECURITY_TMP}
    	elif [[ "${m}" == "/mirror0/director/data" ]]; then
    	    debug "umount_mirror: Calling copy_directory_totemp ${DIR_SECURITY_DIR} ${DIR_SECURITY_TMP}"
            is_mount_on_drbd_device /opt/ibm/director/data
    	    copy_directory_totemp ${DIR_SECURITY_DIR} ${DIR_SECURITY_TMP}
    	elif [[ "${m}" == "/mirror0/director/agentmanager/certs" ]]; then
    	    debug "umount_mirror: Calling copy_directory_totemp ${DIR_AMCERTS_DIR} ${DIR_AMCERTS_TMP}"
            is_mount_on_drbd_device ${DIR_AMCERTS_DIR}
    	    copy_directory_totemp ${DIR_AMCERTS_DIR} ${DIR_AMCERTS_TMP}
    	fi
    	
        cmd_out=`LC_ALL=C umount $m 2>&1`
        urc=$?
        if [ $urc -eq 0 ]; then
        	debug "umount_mirror: unmounted $m"
        	if [[ "${m}" == "/mirror0/director/lwi/conf/overrides" ]]; then
        		# umount worked and it is the overrides dir.
        		# While it is unmounted, copy contents from temp 
        		# back to the base filesystem now
    			debug "umount_mirror: Calling restore_directory_fromtemp ${DIR_OVERRIDES_TMP} ${DIR_OVERRIDES_DIR}"
    			restore_directory_fromtemp ${DIR_OVERRIDES_TMP} ${DIR_OVERRIDES_DIR}
    	    elif [[ "${m}" == "/mirror0/director/lwi/security" ]]; then
    	        debug "umount_mirror: Calling restore_directory_fromtemp ${LWI_SECURITY_TMP} ${LWI_SECURITY_DIR}"
    	        restore_directory_fromtemp ${LWI_SECURITY_TMP} ${LWI_SECURITY_DIR}
    	    elif [[ "${m}" == "/mirror0/director/data" ]]; then
    	        debug "umount_mirror: Calling restore_directory_fromtemp ${DIR_SECURITY_TMP} ${DIR_SECURITY_DIR}"
    	        restore_directory_fromtemp ${DIR_SECURITY_TMP} ${DIR_SECURITY_DIR}
            elif [[ "${m}" == "/mirror0/director/agentmanager/certs" ]]; then
    	        debug "umount_mirror: Calling restore_directory_fromtemp ${DIR_AMCERTS_TMP} ${DIR_AMCERTS_DIR}"
    	        restore_directory_fromtemp ${DIR_AMCERTS_TMP} ${DIR_AMCERTS_DIR}
    		fi
        else
        	 debug "umount_mirror: umount rc=$urc: result:$cmd_out"
        fi
      
    done
}

any_mounts () {
    local mirror=${1}
    
    find_mounts_for_mirror $mirror    

    debug "any_mounts for mirror $mirror, mounts:$MIRROR_MOUNTS:"
    if [ -z "$MIRROR_MOUNTS" ]; then
    	return 0
    else
        return 1
    fi
}

umount_all () {
    local mountpt=${1}
    local name=${2}

    # serialize this function with other processes

	rw_lock write get UmntMir_${name} $name 1000000           # wait "forever" for lock

    # Before attempting umount shutdown any resources that could holds locks
    run_drbd_plugins stop $mountpt
    RESOURCE_STOPPED=$mountpt

    local mounts_left=Y
    local delay_time=0
    local rc
    for i in {1..50}
    do
        # smstatus always needs to be killed
        kill_smstatus 

        # If we've been trying to gracefully shutdown for a while...
        # ...it's time to start killing processes
        if [ $i -gt 8 ]; then
        	if [ $i -gt 16 ]; then
        		force=Y
        	fi
            kill_proc_for_mirror $name $force
        fi
   		
        if [[ "$mounts_left" == "Y" ]]; then
            any_mounts $name
            if [ $? -eq 1 ]; then
                # Unmount any bind mounts        
                umount_mirror $name        
            else
                mounts_left=U
            fi
        fi

        if [[ "$mounts_left" == "U" ]]; then
            mounts_left=N
    	    is_mounted $mountpt
   		    if [ $? -eq 1 ]; then    			
			    cmd_out=`LC_ALL=C umount $mountpt`
                rc=$?
			    debug "umount mirror $mountpt rc=$rc: $cmd_out"
                if [ $rc -ne 0 ]; then
			        mounts_left=U        # try again
                fi
   		    fi 
        fi
        
        if [[ "$mounts_left" == "N" ]]; then
            break
        else
            let "delay_time = i + 1"
            if [ $delay_time -gt 5 ]; then
                deleay_time=5
            fi
            sleep $delay_time
        fi
        
    done
    # Start back up any resources.
    run_drbd_plugins start $mountpt
    RESOURCE_STOPPED=  

	rw_lock write free UmntMir_${name} $name
}

copy_directory_totemp() {
  debug "copy_directory_totemp: entered"
  local from_dir=$1
  local temp_dir=$2
  
  if [ -z $from_dir || -z $temp_dir ]; then
    fatal_error "copy_directory_totemp: 2 parameters required! from_dir=$from_dir; temp_dir=$temp_dir"
    return;
  fi

  if [ ! -e $temp_dir ]; then
    debug "copy_directory_totemp: making temporary directory: $temp_dir"
    mkdir -p $temp_dir
    chmod 700 $temp_dir
  else
  	debug "copy_directory_totemp: clearing temporary directory: $temp_dir"
    # clear all files in the temp directory
    rm -rf $temp_dir/*
  fi
  current_files=`ls $from_dir`
  debug "copy_directory_totemp: current contents of $from_dir:\n$current_files"
  local rc
  cp -af $from_dir/* $temp_dir
  rc=$?
  debug "copy_directory_totemp: cp -af $from_dir/* $temp_dir return code: $rc"
}

restore_directory_fromtemp() {
  local temp_dir=$1
  local dest_dir=$2
  
  if [ -z $temp_dir || -z $dest_dir ]; then
    fatal_error "restore_directory_fromtemp: 2 parameters required! temp_dir=$temp_dir; dest_dir=$dest_dir"
    return;
  fi
  
  # only copy the files if the $dest_dir/.ha_mounted file does not exist or the directory was mounted on a DRBD device
  if [[ ! -e $dest_dir/.ha_unmounted || $MOUNTED_ON_DRBD -eq 1 ]]; then
	if [ ! -e $dest_dir ]; then
  	  debug "restore_directory_fromtemp: making directory: $dest_dir"
  	  mkdir -p $dest_dir
  	  chmod 755 $dest_dir
    else
	  debug "restore_directory_fromtemp: clearing filesystem $dest_dir directory"
  	  rm -rf $dest_dir/*
    fi
    debug "restore_directory_fromtemp: restoring files from $temp_dir to $dest_dir"
    current_files=`ls $temp_dir`
    debug "restore_directory_fromtemp: current contents of $temp_dir:\n$current_files"
    local rc
    cp -af $temp_dir/* $dest_dir
    rc=$?
    debug "restore_directory_fromtemp: cp -af $temp_dir/* $dest_dir return code: $rc"
    touch $dest_dir/.ha_unmounted
  else
    debug "restore_directory_fromtemp: skipping copy to $dest_dir"
  fi
  rm -rf $temp_dir/*
  rc=$?
  debug "restore_directory_fromtemp: rm -rf $temp_dir/* return code: $rc"
}

lock_base() {
    
    # See if the base is locked already by using a file
    if [ ! -e ${DRBD_LOCK_FILE}-base ]; then 
        
        local lock_out=`LC_ALL=C rgmbrreq -o lock IBM.Application:${DRBDB_RSRC_NAME}`
        debug "lock base : $lock_out"
        touch ${DRBD_LOCK_FILE}-base
    fi
}

unlock_base() {
    
    # Only unlock the base if there are no mirror locks
    if [ -e ${DRBD_LOCK_FILE}${mirror}* ]; then
    	debug "Cannot unlock base drbd, cause there are other locks."
    	return
    fi
        
    # See if the base is locked already by using a file
    if [ -e ${DRBD_LOCK_FILE}-base ]; then 
        
        local lock_out=`LC_ALL=C rgmbrreq -o unlock IBM.Application:${DRBDB_RSRC_NAME}`
        debug "unlock base: $lock_out"
        
        rm ${DRBD_LOCK_FILE}-base
    fi
}

are_both_secondary () {
	
    local secondary=`drbdadm state $name | grep "Secondary/Secondary"`
    if [ "x$secondary" == "x"  ]
    then
        return 0
    else
        return 1
    fi

    return 0    
		
}

are_both_inconsistent () {
    
    local inconsistent=`drbdadm state $name | grep "Inconsistent/Inconsistent"`
    if [ "x$inconsistent" == "x"  ]
    then
        return 0
    else
        return 1
    fi

    return 0    
        
}

# The following function is used to check for IBM.ServiceIP addresses that may be in a failed
# offline state and, if so, reset them ... provided they have a DependsOn relationship to the
# replication subnet.
#
# N.B. This function is called when a local DRBD mirror was found to be in a StandAlone state
# and the mirror was just connected to its peer.

check_ServiceIP_failed_offline () {
    local loc_rep_addr=$1
    local OpState
    local RH
    local AgRH
    local TRH
    local Token
    local MC
    local SS
    local rc
    local cmd_out

    debug "check_ServiceIP_failed_offline: local replication address: $loc_rep_addr"

    # Find any floating IP addresses, (constituents) that are failed offline. Given that the local DRBD
    # mirror was in the StandAlone state, odds are that a local interface was offline, which would have
    # led to the IBM.ServiceIP associated with that interface being failed offline. The IBM.ServiceIP
    # may also be failed offline if it detects a problem with the associated interface.

    lsrsrc-api -s "IBM.ServiceIP::ResourceType == 0::OpState::ResourceHandle" 2> /dev/null | \
    while read OutputLine
    do
        OpState=${OutputLine%::*}     # get the OpState value
        RH=${OutputLine#*::}          # get the ResourceHandle
    
        debug "check_ServiceIP_failed_offline: OpState = $OpState   RH = $RH"
        if [ $OpState -eq 3 ]; then

            # this constituent is failed offline. Find its aggregate
    
            AgRH=$(lsrsrc-api -r "$RH"::AggregateResource 2>&1)
            rc=$?
            if [ $rc -ne 0 ]; then
                debug "check_ServiceIP_failed_offline: fetching AgRH, lsrsrc-api rc=$rc, out: $AgRH"
                continue
            fi
    
            # The aggregate is the floating resource, which is the source of a DependsOn
            # relationship to an Equivalency of IBM.Networkinterface resources. Find the
            # resource handle of the target equivalency
            #
            # N.B. By construction of the HA resource model, this relationship only has
            # one target

            Token=$(lsrsrc-api -s "IBM.ManagedRelationship::Source == '$AgRH'::Target" 2>&1)
            rc=$?
            if [ $rc -ne 0 ]; then
                debug "check_ServiceIP_failed_offline: fetching Target, lsrsrc-api rc=$rc, out: $Token"
                continue
            fi

            TRH=${Token#\{}     # Strip off the {}'s (the Target attribute is an array)
            TRH=${TRH%\}}
    
            # using information from the equivalency find the class name of the members
            # and the select string used to find the members.

            Token=$(lsrsrc-api -r "$TRH"::MemberClass::SelectString 2>&1)
            rc=$?
            if [ $rc -ne 0 ]; then
                debug "check_ServiceIP_failed_offline: fetching class/select, lsrsrc-api rc=$rc, out: $Token"
                continue
            fi

            MC=${Token%::*}       # get the value of MemberClass

            # get the select string, which is further restricted to find the resource matching 
            # the local replication address

            SS="(${Token#*::}) && IPAddress == '${loc_rep_addr}'"
    
            if [ "$MC" = "IBM.NetworkInterface" ]; then
                cmd_out=$(lsrsrc-api -s "$MC::$SS::IPAddress" 2>&1)
                rc=$?
                debug "check_ServiceIP_failed_offline: for NI select ${SS}, lsrsrc-api rc=$rc, out: $cmd_out"
                if [ $rc -eq 0 ]; then
                    cmd_out=$(resetrsrc-api -r "$RH" 2>&1)
                    rc=$?
                    debug "check_ServiceIP_failed_offline: for $RH, resetrsrc rc=$rc, out: $cmd_out"
                fi
            else
                debug "check_ServiceIP_failed_offline: unexpected MemberClass value: $MC"
            fi
        fi
    done
}

# Split Brain is detected by DRBD when the peers connect and, during the initial
# protocol handshake, it is determined that both nodes are, or at some point during
# the network interruption were, both primary. DRBD is configured to resolve split
# brain scenarios once the two peers reconnect:
#
#  after-sb-0pri  discard-least-changes
#     split brain is detected and neither node was primary; discard modifications on the
#     node where the fewest changes occurred.
#
#  after-sb-1pri  discard-secondary
#     split brain is detected and one node is primary; make the node that is secondary
#     the split brain "victim", i.e. discard the modifications on the secondary
#
#  after-sb-2pri  disconnect
#     split brain is detected and both nodes are primay; disconnect the peer.
#     This case should not occur since the TSA policy is that only one node may be
#     primary.
#
# However, if one or both peers have a connection state of StandAlone, then an explicit
# connect command must be issued. This function will do so, provided that the replication
# subnet is back online. Once the connection is re-established, the automatic split
# brain resolution can proceed.
#
# N.B. It is likely that DRBD detects loss of communication over the replication subnet
# before RSCT marks the local IBM.NetworkInterface resources on each node as offline.
# Observing that both NI resources are still online is not sufficient. Therefore the
# exit code from the connect command must be checked.

check_and_handle_split_brain () {
	local mirror=$1
    local cstate
    local loc_rep_addr=
    local rem_rep_addr=
    local OpState
    local Count=0
    local drbdadm_out
    local drbdadm_rc

    # Check if we're in a StandAlone state

    cstate=$(drbdadm cstate $mirror 2> /dev/null)

    if [ "$cstate" = "StandAlone" ]; then
        debug "Checking split brain, cstate for $mirror is $cstate"

        if get_value_from_cfg local-replication-address
        then
            loc_rep_addr=$CFG_RETURN_VALUE
        fi

        if get_value_from_cfg remote-replication-address
        then
            rem_rep_addr=$CFG_RETURN_VALUE
        fi

        if [ -n "$loc_rep_addr" -a -n "$rem_rep_addr" ]; then

            for OpState in \
              $(lsrsrc-api -s "IBM.NetworkInterface::IPAddress == '$loc_rep_addr' || IPAddress == '$rem_rep_addr'::OpState" 2> /dev/null)
            do
                if [ $OpState -eq 1 ]; then
                    (( Count++ ))
                fi
            done

            if [ $Count -eq 2 ]; then
                debug "split brain, reconnecting mirror $mirror"
                drbdadm_out=$(drbdadm connect $mirror 2>&1)
                drbdadm_rc=$?
                debug "split brain drbdadm_rc:$drbdadm_rc out:$drbdadm_out"

                if [ $drbdadm_rc -eq 0 ]; then
                    check_ServiceIP_failed_offline $loc_rep_addr
                fi
            fi
        fi
    fi 
	
}

clear_drbd_upgrade_flag () {
    if [ -e $DRBD_UPGRADE_FLAG ]; then
        debug "clear_drbd_upgrade_flag: found upgrade flag"
        rm -f $DRBD_UPGRADE_FLAG
    fi
}

drbd_mutex_on () {
    debug "get write lock for drbd"
	rw_lock write get drbd $1 100000           # wait "forever" for lock
}

drbd_mutex_off () {
    debug "free write lock for drbd"
	rw_lock write free drbd $1
}

drbd_get_read_lock () {
    local rc
    local rsrc=$1
    local timer=$2

    rw_lock read get NameChange $rsrc $timer
    rc=$?

    return $rc
}

drbd_free_read_lock () {
    local rsrc=$1

    rw_lock read free NameChange $rsrc
}

#################################################
#  Main code
#################################################
RC=$TSA_UNKNOWN

# Get the parameters
action=${1} # start, stop, status
resource=${2} # base, primary, mounted
name=${3} # The disk label name (like mirror0)
device=${4} # The device {like /dev/sda5}
mountpt=${5} # The mount point {like /mirror0}
drbddev=${6} # The drbd device (like /dev/drbd0)

DRBD_POST_RESTORE_FORCE_PRIMARY=${DRBD_POST_RESTORE_FORCE_PRIM_PFX}${name}

DRBDB_START_CMD="/etc/rc.d/drbd start"
DRBDP_START_CMD_OVERWRITE="drbdadm -- --overwrite-data-of-peer primary $name"
DRBDP_START_CMD="drbdadm primary $name"
DRBDB_STOP_CMD="/etc/rc.d/drbd stop"
DRBDP_STOP_CMD="drbdadm secondary $name"

debug_tsa "DRBD Action: $action Resource: $resource name: $name"

case ${action} in
start)
    record_tsa_action $action drbd-${resource}-$name "-"
################### START ############################
    wait_for_lock=$DRBD_MON_START_TIMEOUT

	if drbd_get_read_lock rt_${resource}_${name} $(( wait_for_lock * 3 / 4 )) 1; then
		case ${resource} in
		base)
            if [ ! -e $DRBD_NOSTART_FLAG ]; then
			    set_drbd_starting_flag
			
			    modprobe_drbd
			
			    umount_all_mirrors

                # Create the meta data if it doesn't already exist
                crt_all_metadata        
             
                any_mirror_mounts 
                if [ $? -eq 0 ]; then
                    # Start up			    
		           $DRBDB_START_CMD
		           sm_sys_log "DRBD started"
		           RC=${TSA_UNKNOWN}  
                fi
            
                reset_drbd_starting_flag
            else
                RC=${TSA_OFFLINE}     #  do no start during upgrade of active node
            fi
		    ;;
        primary)    
            # Make this node the primary node
            
            for i in 1 2 3 4 5
            do
                if [ -e /proc/drbd ]; then
                    break
                else
                    debug "/proc/drbd not found. drbd base must not be online!"
                    sleep 8
                fi
            done
            
            drbd_mutex_on ${resource}
            
            # check if drbd base is starting, and if so, wait until it has started
            for j in 1 2 3 4 5 6 7 8 9 10
            do
                if [ ! -e $DRBD_STARTING_FILE ]; then
                    break
                else
                    debug "drbd base is still starting, so wait for 5 seconds then check again"
                    sleep 5
                fi
            done
            
            mirrors_state=`drbd-overview  2>&1`
            debug "Start primary $drbddev, mirror states:\n$mirrors_state"
            if [ -e $DRBD_POST_RESTORE_FORCE_PRIMARY ]; then
                for j in {1..10}
                do
                    cmd_out=`$DRBDP_START_CMD_OVERWRITE 2>&1`
                    temprc=$?
                    debug "Attempt $j to start primary (overwrite) $drbddev: start cmd rc: $temprc : cmdout: $cmd_out"
                    if [ $temprc -eq 0 ]; then
                        rm -f $DRBD_POST_RESTORE_FORCE_PRIMARY
                        break
                    fi
                    sleep 5
                done
                debug "primary $drbddev start (overwrite) rc:$temprc out:$cmd_out"
            else
                for j in {1..10}
                do
                    cmd_out=`$DRBDP_START_CMD 2>&1`
                    temprc=$?
                    debug "Attempt $j to start primary $drbddev: start cmd rc: $temprc : cmdout: $cmd_out"
                    if [ $temprc -eq 0 -o $temprc -eq 17 ]; then
                        break
                    fi
                    sleep 5
                done
                if [ $temprc -eq 17 ]; then
            	    # 17 means both nodes are not uptodate        
                    debug "primary $drbddev started with overwrite"
                    cmd_out=`$DRBDP_START_CMD_OVERWRITE 2>&1`
                    temprc=$?
                fi
                debug "primary $drbddev start rc:$temprc out:$cmd_out"
            fi

            # Check that we're really primary.        
            is_primary
            temprc=$?
            debug "primary $drbddev: is_primary check returned: $temprc" 
            if [ $temprc -eq 1  ]; then
		        
		        #cale - added parameter of mirror name
		        check_invalidate_flag $name
		        clear_drbd_upgrade_flag
		        
		        sm_sys_log "DRBD $drbddev Primary started"
		        RC=0
            else
                RC=${TSA_OFFLINE}
            fi
            drbd_mutex_off ${resource}
            ;;
        mounted)
	        rw_lock write get StartMnt_${name} $name 1000000           # wait "forever" for lock

        	is_mounted $drbddev
        	if [ $? -eq 0 ]; then    
        		debug "Start: Mounting $drbddev on $mountpt"
            	cmd_out=`LC_ALL=C mount -o rw $drbddev $mountpt`
            	debug "$cmd_out"

                run_drbd_plugins bindmounts $mountpt
                
                # A plugin script may have run an updatefstab during
                # the previous call.  This can result in a 'mount -a' 
                # being done, which can result in /opt/ibm/director/data/updateslib
                # being mounted before /updateslib is mounted.  That is bad.
                if [[ "${mountpt}" == "/mirror0" ]]; then
                	ensure_updateslib_not_mounted_incorrectly
                fi
            	
            	mount_mirror_bind_mounts $mountpt
        	else
        	   debug "Start: Already mounted $drbddev"    	   
        	fi
        	
        	run_drbd_plugins start $mountpt

	        rw_lock write free StartMnt_${name} $name
        	
            sm_sys_log "DRBD Mounted started"
        	
            RC=0
        
            ;;        
        *)
            fatal_error "Bad parameter to $PROGNAME.  resource is: ${resource}"
            ;;        
        esac

        drbd_free_read_lock rt_${resource}_${name}
	else
        fatal_error "Cannot acquire read lock. resource is: ${resource}"
    fi
    
    ;;
stop)
    record_tsa_action $action drbd-${resource}-$name "-"
################### STOP ############################
    wait_for_lock=$DRBD_MON_STOP_TIMEOUT

	if drbd_get_read_lock op_${resource}_${name} $(( wait_for_lock * 3 / 4 )) 1; then
        case ${resource} in
        base)
        
            # Don't stop while syncing
            wait_while_syncing
            # It should be done syncing, but it might have timed out
            #any_syncing
            #if [ $? -eq 0 ]; then
                $DRBDB_STOP_CMD
            
                modprobe_r_drbd
            
                sm_sys_log "DRBD stopped"
            #else
            #    debug "drbd base not stopped...sync in progress"
            #fi        
            
            
		    RC=0
            ;;
        primary)
            # There shouldn't be any bind mounts left, but incase there is:
            umount_all $mountpt $name      
        
            # Make this node the secondary since we're stopping it
            cmd_out=`$DRBDP_STOP_CMD 2>&1`
            debug "primary stop rc:$? out:$cmd_out"
            
            sm_sys_log "DRBD Primary stopped"
            RC=0

            ;;
        mounted)
            umount_all $mountpt $name
            
			sm_sys_log "DRBD Mounted stopped"    
            RC=0
            ;;        
        *)
            fatal_error "Bad parameter to $PROGNAME. resource is: ${resource}"
            ;;        
        esac

        drbd_free_read_lock op_${resource}_${name}
	else
        fatal_error "Cannot acquire read lock. resource is: ${resource}"
    fi

    ;;

status)
################### STATUS ############################
    if [ "${resource}" = "sync" ]; then
        wait_for_lock=20
    else
        wait_for_lock=$DRBD_MON_CMD_TIMEOUT
    fi

	if drbd_get_read_lock us_${resource}_${name} $(( wait_for_lock * 3 / 4 )) $TSA_UNKNOWN; then
        case ${resource} in
        base)    
		    status_out=`/etc/rc.d/drbd status`
		    statusrc=$?
		    debug_tsa "drbd status returned:$statusrc"
		    case $statusrc in
		        0) #Active
		            RC=${TSA_ONLINE}	        
		            any_mirrors=`echo $status_out | grep mirror`
		            if [ -z "$any_mirrors" ]; then
                        # No mirrors are listed!                                       
                        debug "status_out:$status_out"
                        # Skip the checking for no mirrors file if drbd base is still starting
                        if [ ! -e $DRBD_STARTING_FILE ]; then
                        	if [ -e $NO_MIRRORS_FILE ]; then
                            	# We've seen this before, so unload drbd.
                            	debug "No mirrors found in base status. Stopping base"
                            	debug "status_out:$status_out"
                            	$DRBDB_STOP_CMD
                            	modprobe_r_drbd
                            	reset_no_mirrors_flag
                            	RC=${TSA_OFFLINE}
                        	else
                            	# Just record it and see if it happens again.
                            	debug "No mirrors found in base status. Setting Flag file."
                            	set_no_mirrors_flag
                        	fi
                        fi
                    else
                        # We found a mirror so delete our flag file
                        reset_no_mirrors_flag
                    fi
		            ;;
		        *) #Inactive
		            reset_no_mirrors_flag
		            RC=${TSA_OFFLINE}
		            ;;
		    esac
            ;;
        primary)
            is_primary
		    if [ $? -eq 0  ]; then
		    	###
		    	# We're the secondary
		    	###
		    	
                # if neither node is Primary
                are_both_secondary
                if [ $? -eq 1 ]; then
                	debug "Both drbd nodes are secondary.  Handle any locks"
                    if_syncing_clear_flag $name
                fi

		        RC=${TSA_OFFLINE}
		    else
		        RC=${TSA_ONLINE}
                # If we're the primary then do our duties
                	
                check_invalidate_flag $name
            fi
            
            check_and_handle_split_brain $name
                
            ;;
        mounted)        
	        if rw_lock write get StartMnt_${name} $name $(( DRBD_MON_CMD_PERIOD / 2 ))
            then
                is_mounted $mountpt
                if [ $? -eq 0  ]; then
                    RC=${TSA_OFFLINE}
                else
                    # Before we return status, check if it should be mounted
                    is_primary
        	        if [ $? -eq 0  ]; then
        	    	    debug "drbd mounted status: mounted when it should not be; unmounting"
                        umount_all $mountpt $name	    
                        RC=${TSA_OFFLINE}
                    else
                        # On FSM, if the backing store HDD has failed or is removed
                        # DRBD doesn't seem to know about it.  Testing has shown that
                        # if we force a write to the mount that an I/O error results, and 
                        # then it works.  So, let's do that.  Passing mount point, e.g. /dump
                        ensure_backingstore $mountpt
                        RC=${TSA_ONLINE}
        	        fi
            
                fi
	            rw_lock write free StartMnt_${name} $name
            else
                debug "Timed out getting StartMnt_${name} lock"
                RC=${TSA_OFFLINE}
            fi
            
            ;;
        sync)
        
            for j in 1 2 3 4 5 6
            do
                drbdOverviewText=`drbd-overview`
                notFound=`echo "$drbdOverviewText" | grep "not-found"`
                if [ "x$notFound" == "x"  ]
                then
                    debug "drbd sync state full good for $name: \n$drbdOverviewText"
                    break
                else
                    #if drbd reports ??not-found??, wait a few seconds than try again
                    debug "drbd sync state full not-found for $name: \n$drbdOverviewText"
                    sleep 2
                fi
            done

            mirror_state=`echo "$drbdOverviewText" | grep ":$name " | awk '{print $4}'`

            debug "drbd sync state for $name:$mirror_state"
            case "$mirror_state" in        	
            	UpToDate/*)        	   
                    RC=${TSA_ONLINE}
                    ;;
                Inconsistent/Inconsistent)
                    # If both are inconsistent then either can be primary        	
                    RC=${TSA_ONLINE}
                    ;;
                Consistent/*)                
                    # If this node is consistent then there isn't a connection to the other node           
                    RC=${TSA_ONLINE}
                    ;;
                Inconsistent/DUnknown)

                    if [ -e $DRBD_POST_RESTORE_FORCE_PRIMARY ]; then
                        debug "found post restore flag $DRBD_POST_RESTORE_FORCE_PRIMARY, returning online status"
                        RC=${TSA_ONLINE}
                    # Check if this is the upgrade case
                    elif [ -e $DRBD_UPGRADE_FLAG ]; then        	
                        RC=${TSA_ONLINE}
                    else
                	    RC=${TSA_OFFLINE}
                    fi
                    ;;                
            	*)
            	    RC=${TSA_OFFLINE}
            	    ;;
            esac            
            
            ;;
        *)
            fatal_error "Bad parameter to $PROGNAME. resource is: ${resource}"
            ;;        
        esac

        drbd_free_read_lock us_${resource}_${name}
    fi
    ;;
cleanup)
################### CLEANUP ############################
    debug "DRBD ${resource} failed."
;;
*)
    fatal_error "Bad parameter to $PROGNAME. Action is:${action}"
    ;;        
    
esac

record_tsa_action $action drbd-${resource}-$name $RC

exit $RC
